package no.skarasoftwaredevelopment.schooltracker.database;

import android.provider.BaseColumns;

/**
 * This class maintains all the table and column names
 * used by {@code DbHelper}.
 * 
 * @author Tommy
 *
 */
public final class DbContract {
	
	private DbContract() {}
	
	public static abstract class Course implements BaseColumns {
		public static final String TABLE_NAME = "course";
		public static final String COLUMN_NAME_CODE = "code";
		public static final String COLUMN_NAME_NAME = "name";
		public static final String COLUMN_NAME_NORWEGIAN_NAME = "norwegian_name";
		public static final String COLUMN_NAME_NEW_NORWEGIAN_NAME = "new_norwegian_name";
		public static final String COLUMN_NAME_ENGLISH_NAME = "english_name";
		public static final String COLUMN_NAME_VERSION_CODE = "version_code";
	}
	
	public static abstract class Room implements BaseColumns {
		public static final String TABLE_NAME = "room";
		public static final String COLUMN_NAME_ID = "id";
		public static final String COLUMN_NAME_NR = "nr";
		public static final String COLUMN_NAME_NAME = "name"; // location
		public static final String COLUMN_NAME_TYPE = "type";
		public static final String COLUMN_NAME_BUILDING_ID = "building_id";
		public static final String COLUMN_NAME_FLOOR_ID ="floor_id";
		public static final String COLUMN_NAME_LYDIA_CODE = "lydia_code";
		public static final String COLUMN_NAME_GEO_CENTER = "geo_center";
		public static final String COLUMN_NAME_FLOOR = "floor";
		public static final String COLUMN_NAME_BOUNDARY_RADIUS = "boundary_radius";
	}
	
	public static abstract class Student implements BaseColumns {
		public static final String TABLE_NAME = "student";
		public static final String COLUMN_NAME_USERNAME = "username"; // not used
		public static final String COLUMN_NAME_EMAIL = "email"; // not used
		public static final String COLUMN_NAME_COINS = "coins";
		public static final String COLUMN_NAME_TAPS = "taps";
		public static final String COLUMN_NAME_EFFORT_ACTIVITIES_TOTAL = "effort_activities_total";
		public static final String COLUMN_NAME_EFFORT_ACTIVITIES_PASSED = "effort_activities_passed";
		public static final String COLUMN_NAME_EFFORT_ACTIVITIES_ATTENDED = "effort_activities_attended";
		public static final String COLUMN_NAME_EFFORT_COURSE_HIGHEST_NAME = "effort_course_highest_name";
		public static final String COLUMN_NAME_EFFORT_COURSE_HIGHEST_ATTENDED = "effort_course_highest_attended";
		public static final String COLUMN_NAME_EFFORT_COURSE_HIGHEST_TOTAL = "effort_course_highest_total";
		public static final String COLUMN_NAME_EFFORT_COURSE_LOWEST_NAME = "effort_course_lowest_name";
		public static final String COLUMN_NAME_EFFORT_COURSE_LOWEST_ATTENDED = "effort_course_lowest_attended";
		public static final String COLUMN_NAME_EFFORT_COURSE_LOWEST_TOTAL = "effort_course_lowest_total";
		public static final String COLUMN_NAME_EFFORT_UP_TO_DATE = "effort_up_to_date";
		public static final String COLUMN_NAME_STUDY_PROGRAM = "study_program";
		public static final String COLUMN_NAME_UUID = "uuid";
		public static final String COLUMN_NAME_COIN_TOTAL = "coin_total";
		public static final String COLUMN_NAME_TIMESTAMP = "timestamp";
	}
	
	public static abstract class StudentCourse implements BaseColumns {
		public static final String TABLE_NAME = "student_course";
		public static final String COLUMN_NAME_STUDENT_ID = "student_id";
		public static final String COLUMN_NAME_COURSE_CODE = "course_code";
	}
	
	public static abstract class StudentCourseEffort implements BaseColumns {
		public static final String TABLE_NAME = "student_course_effort";
		public static final String COLUMN_NAME_STUDENT_ID= "student_id";
		public static final String COLUMN_NAME_COURSE_CODE = "course_code";
		public static final String COLUMN_NAME_ACTIVITIES_TOTAL = "activities_total";
		public static final String COLUMN_NAME_ACTIVITIES_PASSED = "activities_passed";
		public static final String COLUMN_NAME_ACTIVITIES_ATTENDED = "activities_attended";
		public static final String COLUMN_NAME_LECTURES_PASSED = "lectures_passed";
		public static final String COLUMN_NAME_LECTURES_ATTENDED = "lectures_attended";
		public static final String COLUMN_NAME_LECTURES_FULL = "lectures_full";
		public static final String COLUMN_NAME_EXERCISES_PASSED = "exercises_passed";
		public static final String COLUMN_NAME_EXERCISES_ATTENDED = "exercises_attended";
		public static final String COLUMN_NAME_UP_TO_DATE = "up_to_date";
	}
	
	public static abstract class StudentPosition implements BaseColumns {
		public static final String TABLE_NAME = "student_position";
		public static final String COLUMN_NAME_STUDENT_ID = "student_id";
		public static final String COLUMN_NAME_ATTENDANCE_ID = "attendance_id";
		public static final String COLUMN_NAME_TIMESTAMP = "timestamp";
		public static final String COLUMN_NAME_LONGITUDE = "longitude";
		public static final String COLUMN_NAME_LATITUDE = "latitude";
		public static final String COLUMN_NAME_FLOOR = "floor";
		public static final String COLUMN_NAME_INSIDE = "inside";
		public static final String COLUMN_NAME_PROVIDER = "provider";
	}
	
	public static abstract class StudentAchievement implements BaseColumns {
		public static final String TABLE_NAME = "student_achievement";
		public static final String COLUMN_NAME_STUDENT_ID= "student_id";
		public static final String COLUMN_NAME_ACHIEVEMENT_ID = "achievement_id";
		public static final String COLUMN_NAME_TIMESTAMP = "timestamp";
	}
	
	public static abstract class StudentItem implements BaseColumns {
		public static final String TABLE_NAME = "student_item";
		public static final String COLUMN_NAME_TYPE = "type";
		public static final String COLUMN_NAME_NAME = "name";
		public static final String COLUMN_NAME_VALUE = "value";
		public static final String COLUMN_NAME_USED = "used";
		public static final String COLUMN_NAME_DIFFICULTY = "difficulty";
		public static final String COLUMN_NAME_TIMESTAMP = "timestamp";
		public static final String COLUMN_NAME_STUDENT_ID = "student_id";
	}
	
	public static abstract class Schedule implements BaseColumns {
		public static final String TABLE_NAME = "schedule";
		public static final String COLUMN_NAME_COURSE_CODE = "course_code";
	}

	public static abstract class CourseActivity implements BaseColumns {
		public static final String TABLE_NAME = "course_activity";
		public static final String COLUMN_NAME_ACTIVITY_ID = "activity_id";
		public static final String COLUMN_NAME_COURSE_VERSION_CODE = "course_version_code";
		public static final String COLUMN_NAME_ACTIVITY_DESCRIPTION = "activity_description";
		public static final String COLUMN_NAME_STUDY_PROGRAMS = "study_programs";
		public static final String COLUMN_NAME_COURSE_CODE = "course_code";
	}
	
	public static abstract class Timetable implements BaseColumns {
		public static final String TABLE_NAME = "timetable";
		public static final String COLUMN_NAME_DAY_NUMBER = "day_number";
		public static final String COLUMN_NAME_START_TIME = "start_time";
		public static final String COLUMN_NAME_END_TIME = "end_time";
		public static final String COLUMN_NAME_ACTIVITY_ID = "activity_id";
	}
	
	public static abstract class TimetableRoom implements BaseColumns {
		public static final String TABLE_NAME = "timetable_room";
		public static final String COLUMN_NAME_TIMETABLE_ID = "timetable_id";
		public static final String COLUMN_NAME_LYDIA_CODE = "lydia_code";
	}
	
	public static abstract class Attendance implements BaseColumns {
		public static final String TABLE_NAME = "attendance";
		public static final String COLUMN_NAME_START_DATE = "start_date";
		public static final String COLUMN_NAME_END_DATE = "end_date";
		public static final String COLUMN_NAME_DURATION = "duration";
		public static final String COLUMN_NAME_ATTENDED = "has_attended";
		public static final String COLUMN_NAME_PASSED = "has_passed";
		public static final String COLUMN_NAME_TIMETABLE_ID = "timetable_id";
		public static final String COLUMN_NAME_STUDENT_ID = "student_id";
		public static final String COLUMN_NAME_TIMESTAMP = "timestamp";
	}
	
	public static abstract class Achievement implements BaseColumns {
		public static final String TABLE_NAME = "achievement";
		public static final String COLUMN_NAME_ID = "id";
		public static final String COLUMN_NAME_CATEGORY = "category";
		public static final String COLUMN_NAME_NAME= "name";
		public static final String COLUMN_NAME_DESCRIPTION = "description";
		public static final String COLUMN_NAME_VALUE = "value";
	}
	
	public static abstract class ShopItem implements BaseColumns {
		public static final String TABLE_NAME = "shop_item";
		public static final String COLUMN_NAME_TYPE = "type";
		public static final String COLUMN_NAME_NAME = "name";
		public static final String COLUMN_NAME_DESCRIPTION = "description";
		public static final String COLUMN_NAME_PRICE = "price";
		public static final String COLUMN_NAME_AMOUNT = "amount";
		public static final String COLUMN_NAME_REQUIREMENT = "requirement";
	}
	
}