package no.skarasoftwaredevelopment.schooltracker.database;

import java.io.Serializable;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler.AchievementType;
import no.skarasoftwaredevelopment.schooltracker.handlers.FormatHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.AttendanceStats;
import no.skarasoftwaredevelopment.schooltracker.pojo.Chest;
import no.skarasoftwaredevelopment.schooltracker.pojo.Coord;
import no.skarasoftwaredevelopment.schooltracker.pojo.Course;
import no.skarasoftwaredevelopment.schooltracker.pojo.CourseActivity;
import no.skarasoftwaredevelopment.schooltracker.pojo.IceAxe;
import no.skarasoftwaredevelopment.schooltracker.pojo.Item;
import no.skarasoftwaredevelopment.schooltracker.pojo.Item.ItemType;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData.PositionProvider;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.Schedule;
import no.skarasoftwaredevelopment.schooltracker.pojo.ShopItem;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudentCourseEffort;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudentPosition;
import no.skarasoftwaredevelopment.schooltracker.pojo.Timetable;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.ActivityType;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteConstraintException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class DbHelper extends SQLiteOpenHelper implements Serializable {

	public static final String TAG = "DbHelper";
	public static final int DATABASE_VERSION = 1;
    public static final String DATABASE_NAME = "SchoolTracker.db";

	private static final long serialVersionUID = -526354955773622749L;
	private static final String PRIMARY_KEY = " PRIMARY KEY";
	private static final String UNIQUE = " UNIQUE";
	private static final String TEXT_TYPE = " TEXT";
	private static final String INTEGER_TYPE = " INTEGER";
	private static final String REAL_TYPE = " REAL";
	private static final String DATE_TYPE = " DATETIME DEFAULT (datetime('now','localtime'))";
	private static final String COMMA_SEP = ",";
	
	private static final String SQL_CREATE_COURSE = 
			"CREATE TABLE " + DbContract.Course.TABLE_NAME + " (" +
			DbContract.Course._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_CODE + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_NAME + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_NORWEGIAN_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_NEW_NORWEGIAN_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_ENGLISH_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Course.COLUMN_NAME_VERSION_CODE + INTEGER_TYPE +
			")";
	
	private static final String SQL_CREATE_ROOM =
			"CREATE TABLE " + DbContract.Room.TABLE_NAME + " (" +
			DbContract.Room._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_ID + INTEGER_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_NR + TEXT_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_NAME + TEXT_TYPE + COMMA_SEP + // location
			DbContract.Room.COLUMN_NAME_TYPE + TEXT_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_BUILDING_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_FLOOR_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_LYDIA_CODE + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_GEO_CENTER + TEXT_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_FLOOR + REAL_TYPE + COMMA_SEP +
			DbContract.Room.COLUMN_NAME_BOUNDARY_RADIUS + INTEGER_TYPE +
			")";
	
	private static final String SQL_CREATE_STUDENT = 
			"CREATE TABLE " + DbContract.Student.TABLE_NAME + " (" +
			DbContract.Student._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_USERNAME + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EMAIL + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_COINS + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_TAPS + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_TOTAL + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_PASSED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_TOTAL + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_TOTAL + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_EFFORT_UP_TO_DATE + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_STUDY_PROGRAM + TEXT_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_UUID + TEXT_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_COIN_TOTAL + INTEGER_TYPE + COMMA_SEP +
			DbContract.Student.COLUMN_NAME_TIMESTAMP + DATE_TYPE +
			")";
	
	private static final String SQL_CREATE_STUDENT_COURSE = 
			"CREATE TABLE " + DbContract.StudentCourse.TABLE_NAME + " (" +
			DbContract.StudentCourse._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + TEXT_TYPE + UNIQUE + COMMA_SEP +
			DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + ") " +
		    "REFERENCES " + DbContract.Course.TABLE_NAME + "(" + DbContract.Course.COLUMN_NAME_CODE + ") " +
		    "FOREIGN KEY (" + DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ")" +
			")";
	
	
	private static final String SQL_CREATE_STUDENT_COURSE_EFFORT =
			"CREATE TABLE " + DbContract.StudentCourseEffort.TABLE_NAME + " (" +
			DbContract.StudentCourseEffort._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE + TEXT_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_TOTAL + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_PASSED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_PASSED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_FULL + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_PASSED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentCourseEffort.COLUMN_NAME_UP_TO_DATE + INTEGER_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ") " +
		    "FOREIGN KEY (" + DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE + ") " +
		    "REFERENCES " + DbContract.Course.TABLE_NAME + "(" + DbContract.Course.COLUMN_NAME_CODE + ")" +
			")";
	
	private static final String SQL_CREATE_STUDENT_POSITION = 
			"CREATE TABLE " + DbContract.StudentPosition.TABLE_NAME + " (" +
			DbContract.StudentPosition._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_TIMESTAMP + DATE_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_LONGITUDE + TEXT_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_LATITUDE + TEXT_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_FLOOR + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_INSIDE + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentPosition.COLUMN_NAME_PROVIDER + TEXT_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ") " +
		    "FOREIGN KEY (" + DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + ") " +
		    "REFERENCES " + DbContract.Attendance.TABLE_NAME + "(" + DbContract.Attendance._ID + ")" +
			")";
	
	private static final String SQL_CREATE_STUDENT_ACHIEVEMENT =
			"CREATE TABLE " + DbContract.StudentAchievement.TABLE_NAME + " (" +
			DbContract.StudentAchievement._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.StudentAchievement.COLUMN_NAME_ACHIEVEMENT_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentAchievement.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentAchievement.COLUMN_NAME_TIMESTAMP + DATE_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.StudentAchievement.COLUMN_NAME_ACHIEVEMENT_ID + ") " +
		    "REFERENCES " + DbContract.Achievement.TABLE_NAME + "(" + DbContract.Timetable._ID + ") " +
		    "FOREIGN KEY (" + DbContract.StudentAchievement.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ")" +
			")";
	
	private static final String SQL_CREATE_COURSE_ACTIVITY = 
			"CREATE TABLE " + DbContract.CourseActivity.TABLE_NAME + " (" +
			DbContract.CourseActivity._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.CourseActivity.COLUMN_NAME_COURSE_VERSION_CODE + TEXT_TYPE + COMMA_SEP +
			DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_DESCRIPTION + TEXT_TYPE + COMMA_SEP +
			DbContract.CourseActivity.COLUMN_NAME_STUDY_PROGRAMS + TEXT_TYPE + COMMA_SEP +
			DbContract.CourseActivity.COLUMN_NAME_COURSE_CODE + TEXT_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.CourseActivity.COLUMN_NAME_COURSE_CODE + ") " +
		    "REFERENCES " + DbContract.Course.TABLE_NAME + "(" + DbContract.Course.COLUMN_NAME_CODE + ")" +
			")";

	private static final String SQL_CREATE_TIMETABLE =
			"CREATE TABLE " + DbContract.Timetable.TABLE_NAME + " (" +
			DbContract.Timetable._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Timetable.COLUMN_NAME_DAY_NUMBER + INTEGER_TYPE + COMMA_SEP +
			DbContract.Timetable.COLUMN_NAME_START_TIME + TEXT_TYPE + COMMA_SEP +
			DbContract.Timetable.COLUMN_NAME_END_TIME + TEXT_TYPE + COMMA_SEP +
			DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID + INTEGER_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID + ") " +
		    "REFERENCES " + DbContract.CourseActivity.TABLE_NAME + "(" + DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_ID + ") " +
			")";
	
	private static final String SQL_CREATE_TIMETABLE_ROOM = 
			"CREATE TABLE " + DbContract.TimetableRoom.TABLE_NAME + " (" +
			DbContract.TimetableRoom._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.TimetableRoom.COLUMN_NAME_TIMETABLE_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.TimetableRoom.COLUMN_NAME_LYDIA_CODE + TEXT_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.TimetableRoom.COLUMN_NAME_TIMETABLE_ID + ") " +
		    "REFERENCES " + DbContract.Timetable.TABLE_NAME + "(" + DbContract.Timetable._ID + ") " +
		    "FOREIGN KEY (" + DbContract.TimetableRoom.COLUMN_NAME_LYDIA_CODE + ") " +
		    "REFERENCES " + DbContract.Room.TABLE_NAME + "(" + DbContract.Room.COLUMN_NAME_LYDIA_CODE + ")" +
			")";

	private static final String SQL_CREATE_ATTENDANCE =
			"CREATE TABLE " + DbContract.Attendance.TABLE_NAME + " (" +
			DbContract.Attendance._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_START_DATE + DATE_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_END_DATE + DATE_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_DURATION + INTEGER_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_ATTENDED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_PASSED + INTEGER_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
			DbContract.Attendance.COLUMN_NAME_TIMESTAMP + DATE_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + ") " +
		    "REFERENCES " + DbContract.Timetable.TABLE_NAME + "(" + DbContract.Timetable._ID + ") " +
		    "FOREIGN KEY (" + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ")" +
			")";
	
	private static final String SQL_CREATE_ITEM =
			"CREATE TABLE " + DbContract.StudentItem.TABLE_NAME + " (" +
			DbContract.StudentItem._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_TYPE + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_VALUE + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_USED + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_DIFFICULTY + INTEGER_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_TIMESTAMP + DATE_TYPE + COMMA_SEP +
			DbContract.StudentItem.COLUMN_NAME_STUDENT_ID + INTEGER_TYPE + COMMA_SEP +
		    "FOREIGN KEY (" + DbContract.StudentItem.COLUMN_NAME_STUDENT_ID + ") " +
		    "REFERENCES " + DbContract.Student.TABLE_NAME + "(" + DbContract.Student._ID + ")" +
			")";
	
	private static final String SQL_CREATE_ACHIEVEMENT =
			"CREATE TABLE " + DbContract.Achievement.TABLE_NAME + " (" +
			// we use our own ID here instead the one BaseColumns (_ID) provide since we want
			// to be able to set the integer ourselves instead of using the auto incrementing
			DbContract.Achievement.COLUMN_NAME_ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.Achievement.COLUMN_NAME_CATEGORY + INTEGER_TYPE + COMMA_SEP +
			DbContract.Achievement.COLUMN_NAME_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.Achievement.COLUMN_NAME_DESCRIPTION + TEXT_TYPE + COMMA_SEP +
			DbContract.Achievement.COLUMN_NAME_VALUE + INTEGER_TYPE +
			")";
	
	private static final String SQL_CREATE_SHOP_ITEM =
			"CREATE TABLE " + DbContract.ShopItem.TABLE_NAME + " (" +
			DbContract.ShopItem._ID + INTEGER_TYPE + PRIMARY_KEY + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_TYPE + INTEGER_TYPE + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_NAME + TEXT_TYPE + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_DESCRIPTION + TEXT_TYPE + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_PRICE + INTEGER_TYPE + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_AMOUNT + INTEGER_TYPE + COMMA_SEP +
			DbContract.ShopItem.COLUMN_NAME_REQUIREMENT + INTEGER_TYPE +
			")";

	private static final String SQL_DROP_COURSE =
			"DROP TABLE IF EXISTS " + DbContract.Course.TABLE_NAME;
	private static final String SQL_DROP_ROOM =
			"DROP TABLE IF EXISTS " + DbContract.Room.TABLE_NAME;
	private static final String SQL_DROP_STUDENT =
			"DROP TABLE IF EXISTS " + DbContract.Student.TABLE_NAME;
	private static final String SQL_DROP_STUDENT_COURSE =
			"DROP TABLE IF EXISTS " + DbContract.StudentCourse.TABLE_NAME;
	private static final String SQL_DROP_STUDENT_COURSE_EFFORT =
			"DROP TABLE IF EXISTS " + DbContract.StudentCourseEffort.TABLE_NAME;
	private static final String SQL_DROP_STUDENT_POSITION =
			"DROP TABLE IF EXISTS " + DbContract.StudentPosition.TABLE_NAME;
	private static final String SQL_DROP_COURSE_ACTIVITY =
			"DROP TABLE IF EXISTS " + DbContract.CourseActivity.TABLE_NAME;
	private static final String SQL_DROP_TIMETABLE =
			"DROP TABLE IF EXISTS " + DbContract.Timetable.TABLE_NAME;
	private static final String SQL_DROP_TIMETABLE_ROOM =
			"DROP TABLE IF EXISTS " + DbContract.TimetableRoom.TABLE_NAME;
	private static final String SQL_DROP_ATTENDANCE =
			"DROP TABLE IF EXISTS " + DbContract.Attendance.TABLE_NAME;
	private static final String SQL_DROP_ITEM =
			"DROP TABLE IF EXISTS " + DbContract.StudentItem.TABLE_NAME;
	private static final String SQL_DROP_ACHIEVEMENT =
			"DROP TABLE IF EXISTS " + DbContract.Achievement.TABLE_NAME;
	private static final String SQL_DROP_STUDENT_ACHIEVEMENT =
			"DROP TABLE IF EXISTS " + DbContract.StudentAchievement.TABLE_NAME;
	private static final String SQL_DROP_SHOP_ITEM =
			"DROP TABLE IF EXISTS " + DbContract.ShopItem.TABLE_NAME;
	
	public DbHelper(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}
	
	public void recreateTables(boolean status) {
		SQLiteDatabase db = getWritableDatabase();
		
		if (status) {
			db.execSQL(SQL_DROP_COURSE);
			db.execSQL(SQL_DROP_ROOM);
			db.execSQL(SQL_DROP_STUDENT);
			db.execSQL(SQL_DROP_STUDENT_COURSE);
			db.execSQL(SQL_DROP_STUDENT_COURSE_EFFORT);
			db.execSQL(SQL_DROP_STUDENT_POSITION);
			db.execSQL(SQL_DROP_STUDENT_ACHIEVEMENT);
			db.execSQL(SQL_DROP_COURSE_ACTIVITY);
			db.execSQL(SQL_DROP_TIMETABLE);
			db.execSQL(SQL_DROP_TIMETABLE_ROOM);
			db.execSQL(SQL_DROP_ATTENDANCE);
			db.execSQL(SQL_DROP_ITEM);
			db.execSQL(SQL_DROP_ACHIEVEMENT);
			db.execSQL(SQL_DROP_SHOP_ITEM);

			db.execSQL(SQL_CREATE_COURSE);
			db.execSQL(SQL_CREATE_ROOM);
			db.execSQL(SQL_CREATE_STUDENT);
			db.execSQL(SQL_CREATE_STUDENT_COURSE);
			db.execSQL(SQL_CREATE_STUDENT_COURSE_EFFORT);
			db.execSQL(SQL_CREATE_STUDENT_POSITION);
			db.execSQL(SQL_CREATE_STUDENT_ACHIEVEMENT);
			db.execSQL(SQL_CREATE_COURSE_ACTIVITY);
			db.execSQL(SQL_CREATE_TIMETABLE);
			db.execSQL(SQL_CREATE_TIMETABLE_ROOM);
			db.execSQL(SQL_CREATE_ATTENDANCE);
			db.execSQL(SQL_CREATE_ITEM);
			db.execSQL(SQL_CREATE_ACHIEVEMENT);
			db.execSQL(SQL_CREATE_SHOP_ITEM);
		}
		
		db.close();
	}
	
	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL(SQL_CREATE_COURSE);
		db.execSQL(SQL_CREATE_ROOM);
		db.execSQL(SQL_CREATE_STUDENT);
		db.execSQL(SQL_CREATE_STUDENT_COURSE);
		db.execSQL(SQL_CREATE_STUDENT_COURSE_EFFORT);
		db.execSQL(SQL_CREATE_STUDENT_POSITION);
		db.execSQL(SQL_CREATE_STUDENT_ACHIEVEMENT);
		db.execSQL(SQL_CREATE_COURSE_ACTIVITY);
		db.execSQL(SQL_CREATE_TIMETABLE);
		db.execSQL(SQL_CREATE_TIMETABLE_ROOM);
		db.execSQL(SQL_CREATE_ATTENDANCE);
		db.execSQL(SQL_CREATE_ITEM);
		db.execSQL(SQL_CREATE_ACHIEVEMENT);
		db.execSQL(SQL_CREATE_SHOP_ITEM);
    }

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		onUpgrade(db, oldVersion, newVersion);
	}
	
	public enum DbStatus {
		OK,
		ENTRY_EXISTS,
		INVALID_DATA,
		NOT_INITIALIZED,
		UNKNOWN_ERROR,
		MISSING_ROOMS,
		NO_ACTIVITIES
	}
	
	/**
	 * This returns a string containing the file
	 * path to the location of the database.
	 * Used when the student uploads its database.
	 * 
	 * @return
	 */
	public synchronized String getPath() {
		SQLiteDatabase db = getWritableDatabase();
		String path = db.getPath();
		db.close();
		
		return path;
	}
	
	/*
	 * OTHER METHODS
	 */
	public synchronized DbStatus cacheSchedule(Schedule schedule) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (schedule != null) {
			if (schedule.getActivities() != null) {
				boolean validRooms = true;
				
				// loop through all activities and timetables to check if
				// this course has any rooms not yet added to the webserver's
				// room database which includes gps coordinates
				for (CourseActivity activity : schedule.getActivities()) {
					List<Timetable> timetables = activity.getTimetables();
					
					for (Timetable t : timetables) {
						if (t.getRooms() == null || !areRoomsAdded(t.getRooms())) {
							validRooms = false;
							break;
						}
					}
					
					// stop looping if we already found a room which isn't added
					if (!validRooms) {
						break;
					}
				}
				
				if (validRooms) {
					Student student = getStudent(1); // TODO: get studentId dynamically
					
					for (CourseActivity activity : schedule.getActivities()) {					
						// create new map of values, where column names are the keys
						ContentValues values = new ContentValues();
			
						values.put(DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_ID, activity.getActivityId());
						values.put(DbContract.CourseActivity.COLUMN_NAME_COURSE_VERSION_CODE, activity.getCourseVersionCode());
						values.put(DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_DESCRIPTION, activity.getType().toString());
						
						// convert string list to comma separated string
						String studyPrograms = "";
						List<String> studyProgramsList = activity.getStudyPrograms();
						for (int i = 0; i < studyProgramsList.size(); i++) {
							studyPrograms += studyProgramsList.get(i);
							
							if (i + 1 < studyProgramsList.size()) {
								studyPrograms += ",";
							}
						}
	
						values.put(DbContract.CourseActivity.COLUMN_NAME_STUDY_PROGRAMS, studyPrograms);
						values.put(DbContract.CourseActivity.COLUMN_NAME_COURSE_CODE, schedule.getCourseCode());

						if (studyProgramsList.isEmpty() || studyProgramsList.contains(student.getStudyProgram())) {
							Log.e(TAG, "study program match, continuing..");
							SQLiteDatabase db = getWritableDatabase();
							db.insertOrThrow(DbContract.CourseActivity.TABLE_NAME, null, values);
							db.close();
						
							cacheTimetable(activity.getActivityId(), activity.getTimetables());
						} else {
							Log.e(TAG, "no study program match, canceling..");
						}
					}
					
					status = DbStatus.OK;
				} else {
					status = DbStatus.MISSING_ROOMS;
				}
			} else {
				status = DbStatus.NO_ACTIVITIES;
			}
		} else {
			status = DbStatus.INVALID_DATA;
		}

		return status;
	}
	
	/*
	 * COURSE METHODS
	 */
	
	public synchronized void cacheCourses(List<Course> courses) {
		SQLiteDatabase db = getWritableDatabase();
		
		for (Course c : courses) {
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();

			values.put(DbContract.Course.COLUMN_NAME_CODE, c.getCode());
			values.put(DbContract.Course.COLUMN_NAME_NAME, c.getName());
			values.put(DbContract.Course.COLUMN_NAME_NORWEGIAN_NAME, c.getNorwegianName());
			values.put(DbContract.Course.COLUMN_NAME_NEW_NORWEGIAN_NAME, c.getNewNorwegianName());
			values.put(DbContract.Course.COLUMN_NAME_ENGLISH_NAME, c.getEnglishName());
			values.put(DbContract.Course.COLUMN_NAME_VERSION_CODE, c.getVersionCode());
	
			// insert new row, returns primary key
			db.insert(DbContract.Course.TABLE_NAME, null, values);
		}
		
		db.close();
	}
	
	public synchronized List<Course> getCourses(String query) {
		List<Course> courses = new ArrayList<Course>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Course.TABLE_NAME +
				" WHERE " + DbContract.Course.COLUMN_NAME_CODE +
				" LIKE '%" + query + "%'" +
				" OR " + DbContract.Course.COLUMN_NAME_NAME + " LIKE '%" + query + "%'", null);
		
		while (cursor.moveToNext()) {			
			String code = cursor.getString(1);
			String name = cursor.getString(2);
			String norwegianName = cursor.getString(3);
			String newNorwegianName = cursor.getString(4);
			String englishName = cursor.getString(5);
			int versionCode = cursor.getInt(6);

			courses.add(new Course(code, name, norwegianName, newNorwegianName, englishName, versionCode));
		}
		
		db.close();
		
		return courses;
	}
	
	public synchronized Course getCourse(String courseCode) {
		Course course = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Course.TABLE_NAME +
				" WHERE " + DbContract.Course.COLUMN_NAME_CODE + " = '" + courseCode + "'", null);
		
		if (cursor.moveToFirst()) {			
			String code = cursor.getString(1);
			String name = cursor.getString(2);
			String norwegianName = cursor.getString(3);
			String newNorwegianName = cursor.getString(4);
			String englishName = cursor.getString(5);
			int versionCode = cursor.getInt(6);

			course = new Course(code, name, norwegianName, newNorwegianName, englishName, versionCode);
		}
		
		db.close();
		
		return course;
	}
	
	/*
	 * ATTENDANCE STATS METHODS
	 */

	/**
	 * Gets the total amount of attendances (ignoring if they have come to pass yet
	 * or if the student has attended them) and the total times the student has actually
	 * attended them thus far.
	 * 
	 * @param studentId
	 * @param course
	 * @return
	 */
	public synchronized AttendanceStats getAttendanceStatsDetailed(int studentId, Course course) {
		List<CourseActivity> activities = getCourseActivities(course.getCode());
		List<Attendance> attendances = new ArrayList<Attendance>();
		
		List<Integer> timetableIds = null;
		for (CourseActivity activity : activities) {				
			timetableIds = getTimetableIds(activity.getActivityId());
			
			for (int timetableId : timetableIds) {
				attendances.addAll(getAllStudentAttendances(studentId, timetableId));
			}
		}
		
		// today's date
    	Calendar now = Calendar.getInstance();

		int passedCounter = 0;
		for (Attendance a : attendances) {
			if (a.getEndDate().getTime().before(now.getTime())) {
				passedCounter++;
			}
		}
		
		double percent = ((double) passedCounter / attendances.size()) * 100;
		
		return new AttendanceStats(course, passedCounter, attendances.size(), percent);
	}
	
	public synchronized AttendanceStats getAttendanceStats(int studentId, Course course) {
		List<CourseActivity> activities = getCourseActivities(course.getCode());
		List<Attendance> attendances = new ArrayList<Attendance>();

		List<Integer> timetableIds = null;
		for (CourseActivity activity : activities) {				
			timetableIds = getTimetableIds(activity.getActivityId());
			
			for (int timetableId : timetableIds) {
				attendances.addAll(getStudentAttendances(studentId, timetableId));
			}
		}
		
		int attendedCounter = 0;
		for (Attendance a : attendances) {
			if (a.hasAttended()) {
				attendedCounter++;
			}
		}
		
		double percent = ((double) attendedCounter / attendances.size()) * 100;
		
		return new AttendanceStats(course, attendedCounter, attendances.size(), percent);
	}
	
	public synchronized AttendanceStats getAttendanceStatsType(int studentId, Course course, ActivityType type) {
		List<CourseActivity> activities = getCourseActivities(course.getCode());
		List<Attendance> attendances = getAttendances(activities, studentId, type);
		
		int attendedCounter = 0;
		for (Attendance a : attendances) {
			if (a.hasAttended()) {
				attendedCounter++;
			}
		}
		
		double percent = ((double) attendedCounter / attendances.size()) * 100;
		
		return new AttendanceStats(course, attendedCounter, attendances.size(), percent);
	}
	
	public synchronized AttendanceStats getAttendanceStatsFull(int studentId, Course course, ActivityType type) {
		List<CourseActivity> activities = getCourseActivities(course.getCode());
		List<Attendance> attendances = getAttendances(activities, studentId, type);

		int attendedCounter = 0;
		int attendedFullCounter = 0;
		List<StudentPosition> positions = null;
		for (Attendance a : attendances) {			
			if (a.hasAttended()) {
				attendedCounter++;
				
				positions = getStudentPositions(studentId, a.getId());
				boolean isFull = true;
				for (StudentPosition pos : positions) {
					if (!pos.isInside()) {
						isFull = false;
						break;
					}
				}
				
				if (isFull) {
					attendedFullCounter++;
				}
			}
		}
		
		double percent = 0.0;
		if (attendedCounter > 0) {
			percent = ((double) attendedFullCounter / attendedCounter) * 100;
		}
		
		return new AttendanceStats(course, attendedFullCounter, attendedCounter, percent);
	}
	
	public synchronized AttendanceStats getHighestAttendanceStats(int studentId) {
		Course highestCourse = null;
		int highestAttendanceCount = 0;
		int highestAttendanceTotal = 0;
		double highestAttendancePercent = 0;
		
		List<Course> courses = getStudentCourses(studentId);
		
		List<CourseActivity> activities = null;
		List<Attendance> attendances = null;
		List<Integer> timetableIds = null;
		
		for (Course course : courses) {
			activities = getCourseActivities(course.getCode());
			attendances = new ArrayList<Attendance>();
			
			for (CourseActivity activity : activities) {				
				timetableIds = getTimetableIds(activity.getActivityId());
				
				for (int timetableId : timetableIds) {
					attendances.addAll(getStudentAttendances(studentId, timetableId));
				}
			}
			
			int attendedCounter = 0;
			for (Attendance a : attendances) {
				if (a.hasAttended()) {
					attendedCounter++;
				}
			}
			
			double percent = ((double) attendedCounter / attendances.size()) * 100;
			
			if (percent > highestAttendancePercent) {
				highestCourse = course;
				highestAttendanceCount = attendedCounter;
				highestAttendanceTotal = attendances.size();
				highestAttendancePercent = percent;
			}
		}
		
		return new AttendanceStats(highestCourse, highestAttendanceCount, highestAttendanceTotal, highestAttendancePercent);
	}
	
	public synchronized AttendanceStats getLowestAttendanceStats(int studentId) {
		Course lowestCourse = null;
		int lowestAttendanceCount = 0;
		int lowestAttendanceTotal = 0;
		double lowestAttendancePercent = 100; // can never be more than 100%
		
		List<Course> courses = getStudentCourses(studentId);
		
		List<CourseActivity> activities = null;
		List<Attendance> attendances = null;
		List<Integer> timetableIds = null;
		
		for (Course course : courses) {
			activities = getCourseActivities(course.getCode());
			attendances = new ArrayList<Attendance>();
			
			for (CourseActivity activity : activities) {				
				timetableIds = getTimetableIds(activity.getActivityId());
				
				for (int timetableId : timetableIds) {
					attendances.addAll(getStudentAttendances(studentId, timetableId));
				}
			}
			
			int attendedCounter = 0;
			for (Attendance a : attendances) {
				if (a.hasAttended()) {
					attendedCounter++;
				}
			}
			
			double percent = ((double) attendedCounter / attendances.size()) * 100;
			
			if (percent <= lowestAttendancePercent) {
				lowestCourse = course;
				lowestAttendanceCount = attendedCounter;
				lowestAttendanceTotal = attendances.size();
				lowestAttendancePercent = percent;
			}
		}
		
		return new AttendanceStats(lowestCourse, lowestAttendanceCount, lowestAttendanceTotal, lowestAttendancePercent);
	}
	
	/*
	 * COURSE ACTIVITY METHODS
	 */
	
	public synchronized List<CourseActivity> getCourseActivities(String courseCode) {
		List<CourseActivity> activities = new ArrayList<CourseActivity>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.CourseActivity.TABLE_NAME +
				" WHERE " + DbContract.CourseActivity.COLUMN_NAME_COURSE_CODE + " = '" + courseCode + "'", null);
		
		while (cursor.moveToNext()) {
			String id = cursor.getString(1); // index 0 is the auto-generated primary key
			String version = cursor.getString(2);
			ActivityType type = ActivityType.getType(cursor.getString(3)); // description
			String studyPrograms = cursor.getString(4);
			
			// create a string list of study programs
			List<String> studyProgramsList = new ArrayList<String>();
			String[] list = studyPrograms.split(",");
			for (String s : list) {
				studyProgramsList.add(s);
			}
			
			activities.add(new CourseActivity(id, version, type, studyProgramsList, courseCode));
		}
		
		db.close();
		
		return activities;
	}
	
	public synchronized CourseActivity getCourseActivity(String activityId) {
		CourseActivity activity = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.CourseActivity.TABLE_NAME +
				" WHERE " + DbContract.CourseActivity.COLUMN_NAME_ACTIVITY_ID + " = '" + activityId + "'", null);
		
		if (cursor.moveToFirst()) {
			String version = cursor.getString(2);
			ActivityType type = ActivityType.getType(cursor.getString(3)); // description
			String studyPrograms = cursor.getString(4);
			String courseCode = cursor.getString(5);
			
			// create a string list of study programs
			List<String> studyProgramsList = new ArrayList<String>();
			String[] list = studyPrograms.split(",");
			for (String s : list) {
				studyProgramsList.add(s);
			}
			
			activity = new CourseActivity(activityId, version, type, studyProgramsList, courseCode);
		}
		
		db.close();
		
		return activity;
	}
	
	/*
	 * TIMETABLE METHODS
	 */
	
	private synchronized void cacheTimetable(String activityId, List<Timetable> timetables) {		
		for (Timetable t : timetables) {			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
			
			values.put(DbContract.Timetable.COLUMN_NAME_DAY_NUMBER, t.getDayNumber());
			values.put(DbContract.Timetable.COLUMN_NAME_START_TIME, t.getStartTime());
			values.put(DbContract.Timetable.COLUMN_NAME_END_TIME, t.getEndTime());
			values.put(DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID, activityId);
			
			SQLiteDatabase db = getWritableDatabase();
			long id = db.insertOrThrow(DbContract.Timetable.TABLE_NAME, null, values);
			db.close();
			
			// TODO: ideally, we need to check if the room is among the cached
			// however, since the way the app works now "all" rooms that are
			// attached to courses will also get added
			addTimetableRooms(id, t.getRooms());
			cacheAttendance(id, t.getAttendance());
		}
		
		//return DbStatus.OK;
	}
	
	public synchronized List<Integer> getTimetableIds(String activityId) {
		List<Integer> ids = new ArrayList<Integer>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.Timetable._ID + " FROM " + DbContract.Timetable.TABLE_NAME +
				" WHERE " + DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID + " = '" + activityId + "'", null);

		while (cursor.moveToNext()) {		
			ids.add(cursor.getInt(0));
		}
		
		db.close();
		
		return ids;
	}

	public synchronized List<Integer> getTimetableIds(String activityId, int dayNumber) {
		List<Integer> ids = new ArrayList<Integer>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.Timetable._ID + " FROM " + DbContract.Timetable.TABLE_NAME +
				" WHERE " + DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID + " = '" + activityId + "'" +
				" AND " + DbContract.Timetable.COLUMN_NAME_DAY_NUMBER + " = " + dayNumber, null);

		while (cursor.moveToNext()) {		
			ids.add(cursor.getInt(0));
		}
		
		db.close();
		
		return ids;
	}
	
	public synchronized Timetable getTimetable(int id) {
		Timetable timetable = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Timetable.TABLE_NAME +
				" WHERE " + DbContract.Timetable._ID + " = " + id, null);
		
		if (cursor.moveToFirst()) {
			int dayNumber = cursor.getInt(1);
			String startTime = cursor.getString(2);
			String endTime = cursor.getString(3);
			String activityId = cursor.getString(4);
			
			List<Room> rooms = getTimetableRooms(id);

			timetable = new Timetable(id, dayNumber, startTime, endTime, activityId, rooms);
		}
		
		db.close();
		
		return timetable;
	}
	
	/**
	 * Not in use as of now.
	 * 
	 * @deprecated
	 * @param studentId
	 * @param dayNumber
	 * @return
	 */
	public synchronized List<Timetable> getTimetablesForStudentDate(int studentId, int dayNumber) {
		List<Timetable> timetables = new ArrayList<Timetable>();
		
		List<String> courseCodes = getStudentCourseCodes(studentId);

    	for (String code : courseCodes) {
    		List<CourseActivity> activities = getCourseActivities(code);
    		
    		for (CourseActivity activity : activities) {
    			List<Integer> timetableIds = getTimetableIds(activity.getActivityId(), dayNumber);
    			
    			for (int id : timetableIds) {
    				timetables.add(getTimetable(id));
    			}
    		}
    	}
    	
    	return timetables;
	}
	
	/*
	 * TIMETABLE ROOM METHODS
	 */
	
	private synchronized void addTimetableRooms(long timetableId, List<Room> rooms) {		
		if (timetableId > -1 && !rooms.isEmpty()) {
			SQLiteDatabase db = getWritableDatabase();
			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
	
			values.put(DbContract.TimetableRoom.COLUMN_NAME_TIMETABLE_ID, timetableId);
	
			for (Room room : rooms) {
				// null rooms here is a result of inconsistencies in the NTNU IME API 
				if (room != null) {
					// insert new row, insert returns primary key
					values.put(DbContract.TimetableRoom.COLUMN_NAME_LYDIA_CODE, room.getLydiaCode());
					db.insertOrThrow(DbContract.TimetableRoom.TABLE_NAME, null, values);
				}
			}
			
			db.close();
		}
	}
	
	/*
	 * ATTENDANCE METHODS
	 */
	
	public synchronized void cacheAttendance(long timetableId, List<Attendance> attendances) {
		// TODO: remove me, just here for testing purposes
		//Random rnd = new Random();
		
		// today's date
    	Calendar now = Calendar.getInstance();

		for (Attendance a : attendances) {			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
			
			// we use this when the student first adds a course to give
			// him 100% attendance so it's more about keeping up the
			// good score instead of having to work towards it
			if (now.getTime().after(a.getEndDate().getTime())) {
				a.setPassed(true);
				a.setAttended(true);
				//a.setAttended(rnd.nextBoolean());
			} else {
				a.setAttended(false);
				a.setPassed(false);
			}
			
			values.put(DbContract.Attendance.COLUMN_NAME_START_DATE, FormatHandler.dateFormat.format(a.getStartDate().getTime()));
			values.put(DbContract.Attendance.COLUMN_NAME_END_DATE, FormatHandler.dateFormat.format(a.getEndDate().getTime()));
			values.put(DbContract.Attendance.COLUMN_NAME_DURATION, a.getDuration());
			values.put(DbContract.Attendance.COLUMN_NAME_ATTENDED, a.hasAttended());
			values.put(DbContract.Attendance.COLUMN_NAME_PASSED, a.hasPassed());
			values.put(DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID, timetableId);
			values.put(DbContract.Attendance.COLUMN_NAME_STUDENT_ID, 1); // TODO: get studentId dynamically!
			
			SQLiteDatabase db = getWritableDatabase();
			long id = db.insertOrThrow(DbContract.Attendance.TABLE_NAME, null, values);
			db.close();
			
			// we only create the dummy positions if
			// the attendance has passed
			if (a.hasPassed()) {
				// we attache two dummies to the attendance
				for (int i = 0; i < 2; i++) {
					StudentPosition pos = new StudentPosition(
							1, (int) id, // TODO: get studentId dynamically!
							String.valueOf(Coord.DUMMY_GEO_LON),
							String.valueOf(Coord.DUMMY_GEO_LON),
							0.0, a.hasAttended(), PositionProvider.DUMMY);
					addStudentPosition(pos);
				}
			}
		}
		
		//return DbStatus.OK;
	}
	
	private synchronized List<Attendance> getAttendances(List<CourseActivity> activities, int studentId, ActivityType type) {
		List<Attendance> attendances = new ArrayList<Attendance>();		
		
		List<Integer> timetableIds = null;
		for (CourseActivity activity : activities) {
			if (type.getClass().isAssignableFrom(activity.getType().getClass())) {
				timetableIds = getTimetableIds(activity.getActivityId());
				
				for (int timetableId : timetableIds) {
					attendances.addAll(getStudentAttendances(studentId, timetableId));
				}
			}
		}
		
		return attendances;
	}
	
	public synchronized void updateAttendance(int studentId, int attendanceId, boolean attended, boolean passed) {
		SQLiteDatabase db = getWritableDatabase();
		
		// new timestamp
		Calendar now = Calendar.getInstance();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Attendance.COLUMN_NAME_ATTENDED, attended);
		values.put(DbContract.Attendance.COLUMN_NAME_PASSED, passed);
		values.put(DbContract.Attendance.COLUMN_NAME_TIMESTAMP, FormatHandler.dateFormat.format(now.getTime()));
		
		db.update(
				DbContract.Attendance.TABLE_NAME, values,
				DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId + " AND " +
				DbContract.Attendance._ID + " = " + attendanceId, null);
		
		db.close();
	}
	
	public synchronized int getTotalAttendances(int studentId) {
		int total = -1;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT COUNT(" + DbContract.Attendance._ID + ") FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId, null);
		
		if (cursor.moveToFirst()) {
			total = cursor.getInt(0);
		}
		
		db.close();
		
		return total;
	}
	
	/**
	 * Counts the number of attendances the student has attended
	 * in the period since the app was first installed and now.
	 * 
	 * @param studentId
	 * @return
	 */
	public synchronized int getTotalAttendancesAttended(int studentId) {
		int total = 0;
		
		Student student = getStudent(studentId);
		SQLiteDatabase db = getReadableDatabase();

		Cursor cursor = db.rawQuery(
				"SELECT COUNT(" + DbContract.Attendance._ID + ") FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.Attendance.COLUMN_NAME_ATTENDED + " = 1" +
				" AND " + DbContract.Attendance.COLUMN_NAME_PASSED + " = 1 " +
				" AND " + DbContract.Attendance.COLUMN_NAME_END_DATE + " >= '" + FormatHandler.dateFormat.format(student.getTimestamp()) + "'", null);
		
		if (cursor.moveToFirst()) {
			total = cursor.getInt(0);
		}
		
		db.close();
		
		return total;
	}
	
	public synchronized List<Attendance> getAttendedTotal(int studentId) {
		List<Attendance> attendances = new ArrayList<Attendance>();
		SQLiteDatabase db = getReadableDatabase();
		
		// today's date
    	Calendar now = Calendar.getInstance();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.Attendance.COLUMN_NAME_END_DATE + " <= '" + FormatHandler.dateFormat.format(now.getTime()) + "'", null);
		
		while (cursor.moveToNext()) {
			Calendar startDate = Calendar.getInstance();
			String startDateString = cursor.getString(1);
			
			Calendar endDate = Calendar.getInstance();
			String endDateString = cursor.getString(2);
			
			int duration = cursor.getInt(3);
			
			boolean attended = cursor.getInt(4) > 0;
			boolean passed = cursor.getInt(5) > 0;
			
			try {				
				startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
				endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
			} catch (ParseException e) {}
			
			attendances.add(new Attendance(startDate, endDate, duration, attended, passed));
		}
		
		db.close();
		
		return attendances;
	}
	
	public synchronized List<Integer> getAttendanceIds(int studentId, int timetableId) {
		List<Integer> attendanceIds = new ArrayList<Integer>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = " + timetableId, null);
		
		while (cursor.moveToNext()) {
			int id = cursor.getInt(0);
			attendanceIds.add(id);
		}
		
		db.close();
		
		return attendanceIds;
	}
	
	public synchronized List<Attendance> getAttendances(int studentId) {
		List<Attendance> attendances = new ArrayList<Attendance>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId, null);
		
		while (cursor.moveToNext()) {
			Calendar startDate = Calendar.getInstance();
			String startDateString = cursor.getString(1);
			
			Calendar endDate = Calendar.getInstance();
			String endDateString = cursor.getString(2);
			
			int duration = cursor.getInt(3);
			
			boolean attended = cursor.getInt(4) > 0;
			boolean passed = cursor.getInt(5) > 0;
			
			try {				
				startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
				endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
			} catch (ParseException e) {}
			
			attendances.add(new Attendance(startDate, endDate, duration, attended, passed));
		}
		
		db.close();
		
		return attendances;
	}
	
	/**
	 * Gets all the student attendances for this timetable meaning it does not take
	 * into account if the attendance has passed yet or not.
	 * 
	 * @param studentId
	 * @param timetableId
	 * @return
	 */
	public synchronized List<Attendance> getAllStudentAttendances(int studentId, int timetableId) {
		List<Attendance> attendances = new ArrayList<Attendance>();
		SQLiteDatabase db = getReadableDatabase();

		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = " + timetableId, null);
			
		while (cursor.moveToNext()) {
			int attendanceId = cursor.getInt(0);
			
			Calendar startDate = Calendar.getInstance();
			String startDateString = cursor.getString(1);
			
			Calendar endDate = Calendar.getInstance();
			String endDateString = cursor.getString(2);
			
			int duration = cursor.getInt(3);
			
			boolean attended = cursor.getInt(4) > 0; // 1 = attended
			boolean passed = cursor.getInt(5) > 0; // 1 = passed

			try {				
				startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
				endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
			} catch (ParseException e) {}
			
			attendances.add(new Attendance(attendanceId, startDate, endDate, duration, attended, passed));
		}
			
		db.close();
		
		return attendances;
	}
	
	/**
	 * Gets all the student attendances belonging to a specific timetable for a
	 * given student after that has taken place before or equal to todays date.
	 * 
	 * @param studentId
	 * @param timetableId
	 * @return
	 */
	public synchronized List<Attendance> getStudentAttendances(int studentId, int timetableId) {
		List<Attendance> attendances = new ArrayList<Attendance>();
		SQLiteDatabase db = getReadableDatabase();
		
		// today's date
    	Calendar now = Calendar.getInstance(new Locale("NO"));

		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
				" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = " + timetableId +
				" AND " + DbContract.Attendance.COLUMN_NAME_END_DATE + " <= '" + FormatHandler.dateFormat.format(now.getTime()) + "'", null);
			
		while (cursor.moveToNext()) {
			int attendanceId = cursor.getInt(0);
			
			Calendar startDate = Calendar.getInstance();
			String startDateString = cursor.getString(1);
			
			Calendar endDate = Calendar.getInstance();
			String endDateString = cursor.getString(2);
			
			int duration = cursor.getInt(3);
			
			boolean attended = cursor.getInt(4) > 0; // 1 = attended
			boolean passed = cursor.getInt(5) > 0; // 1 = passed

			try {				
				startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
				endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
			} catch (ParseException e) {}
			
			attendances.add(new Attendance(attendanceId, startDate, endDate, duration, attended, passed));
		}
			
		db.close();
		
		return attendances;
	}
	
	public synchronized List<Attendance> getStudentCourseAttendances(int studentId, String courseCode, int dayNumber, int weekNumber) {
		// we need to find all relevant timetable ids for the given day of week, a given student, and a given course
		List<Integer> timetableIds = new ArrayList<Integer>();

		List<CourseActivity> activities = getCourseActivities(courseCode);
		
		for (CourseActivity activity : activities) {				
			timetableIds.addAll(getTimetableIds(activity.getActivityId(), dayNumber));
		}
		
		// when we have the timetable ids we can then find all of this weeks attendences on this day, for the given student
		List<Attendance> attendances = new ArrayList<Attendance>();

		for (int id : timetableIds) {
			SQLiteDatabase db = getReadableDatabase();
			
			Cursor cursor = db.rawQuery(
					"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
					" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
					" AND " + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = " + id +
					" ORDER BY " + DbContract.Attendance.COLUMN_NAME_START_DATE, null);
			
			while (cursor.moveToNext()) {
				int attendanceId = cursor.getInt(0);
				
				Calendar startDate = Calendar.getInstance();
				String startDateString = cursor.getString(1);
				
				Calendar endDate = Calendar.getInstance();
				String endDateString = cursor.getString(2);
				
				int duration = cursor.getInt(3);
				
				boolean attended = cursor.getInt(4) > 0; // 1 = attended
				boolean passed = cursor.getInt(5) > 0; // 1 = passed

				try {				
					startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
					endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
				} catch (ParseException e) {
				}
				
				// check if the scheduled attendance is the same as the current week
				if (weekNumber == startDate.get(Calendar.WEEK_OF_YEAR)) {
					// TODO: there is probably a better way to do this to ensure that
					// we get the activityDescription, courseCode, courseName, and the rooms
					Timetable timetable = getTimetable(id);
					CourseActivity activity = getCourseActivity(timetable.getActivityId());
					Course course = getCourse(activity.getCourseCode());
					
					attendances.add(new Attendance(
							attendanceId, startDate, endDate, duration,
							attended, passed, course.getCode(), course.getName(),
							activity.getType(), timetable.getRooms()));
				}
			}
			
			db.close();
		}
		
		Collections.sort(attendances, new AttendanceComparator());
		return attendances;
	}
	
	public synchronized List<Attendance> getStudentAttendances(int studentId, int dayNumber, int weekNumber) {
		// we need to find all relevant timetable ids for the given day of week and a given student
		List<String> codes = getStudentCourseCodes(studentId);
		List<Integer> timetableIds = new ArrayList<Integer>();
		
		for (String code : codes) {
			List<CourseActivity> activities = getCourseActivities(code);
			
			for (CourseActivity activity : activities) {				
				timetableIds.addAll(getTimetableIds(activity.getActivityId(), dayNumber));
			}
		}
		
		// when we have the timetable ids we can then find all of this weeks attendences on this day, for the given student
		List<Attendance> attendances = new ArrayList<Attendance>();

		for (int id : timetableIds) {
			SQLiteDatabase db = getReadableDatabase();
			
			Cursor cursor = db.rawQuery(
					"SELECT * FROM " + DbContract.Attendance.TABLE_NAME +
					" WHERE " + DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = " + studentId +
					" AND " + DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = " + id +
					" ORDER BY " + DbContract.Attendance.COLUMN_NAME_START_DATE, null);
			
			while (cursor.moveToNext()) {
				int attendanceId = cursor.getInt(0);
				
				Calendar startDate = Calendar.getInstance();
				String startDateString = cursor.getString(1);
				
				Calendar endDate = Calendar.getInstance();
				String endDateString = cursor.getString(2);
				
				int duration = cursor.getInt(3);
				
				boolean attended = cursor.getInt(4) > 0; // 1 = attended
				boolean passed = cursor.getInt(5) > 0; // 1 = passed

				try {				
					startDate.setTime(FormatHandler.dateFormat.parse(startDateString));
					endDate.setTime(FormatHandler.dateFormat.parse(endDateString));
				} catch (ParseException e) {
				}
				
				// check if the scheduled attendance is the same as the current week
				if (weekNumber == startDate.get(Calendar.WEEK_OF_YEAR)) {
					// TODO: there is probably a better way to do this to ensure that
					// we get the activityDescription, courseCode, courseName, and the rooms
					Timetable timetable = getTimetable(id);
					CourseActivity activity = getCourseActivity(timetable.getActivityId());
					Course course = getCourse(activity.getCourseCode());
					
					Attendance attendance = new Attendance(
							attendanceId, startDate, endDate, duration,
							attended, passed, course.getCode(), course.getName(),
							activity.getType(), timetable.getRooms());
					
					if (!attendance.hasPassed() && !attendance.hasAttended()) {
						attendance.setInside(isLastStudentPositionInside(studentId, attendance.getId()));
					}
					
					attendances.add(attendance);
				}
			}
			
			db.close();
		}
		
		Collections.sort(attendances, new AttendanceComparator());
		return attendances;
	}
	
	/**
	 * Used to sort attendances according to their start date.
	 * Only used in the ScehduleFragment as of now.
	 */
	private class AttendanceComparator implements Comparator<Attendance> {
	    @Override
	    public int compare(Attendance o1, Attendance o2) {
	        return o1.getStartDate().compareTo(o2.getStartDate());
	    }
	}
	
	/*
	 * ROOM METHODS
	 */
	
	public synchronized void cacheRooms(List<Room> rooms) {
		SQLiteDatabase db = getWritableDatabase();
		
		for (Room r : rooms) {
			if (r != null) {
				// create new map of values, where column names are the keys
				ContentValues values = new ContentValues();
				values.put(DbContract.Room.COLUMN_NAME_ID, r.getId());
				values.put(DbContract.Room.COLUMN_NAME_NR, r.getNr());
				values.put(DbContract.Room.COLUMN_NAME_NAME, r.getName());
				values.put(DbContract.Room.COLUMN_NAME_TYPE, r.getType());
				values.put(DbContract.Room.COLUMN_NAME_BUILDING_ID, r.getBuildingId());
				values.put(DbContract.Room.COLUMN_NAME_FLOOR_ID, r.getFloorId());
				values.put(DbContract.Room.COLUMN_NAME_LYDIA_CODE, r.getLydiaCode());
				
				// since SQLite 3 doesn't have a spatial data type called Point like
				// MySQL does (or anything resembling that), we just save it as a
				// comma-separated text string
				Coord coord = r.getGeoCenter();
				String coordText = coord.getLongitude() + "," + coord.getLatitude();
				values.put(DbContract.Room.COLUMN_NAME_GEO_CENTER, coordText);
				
				values.put(DbContract.Room.COLUMN_NAME_FLOOR, r.getFloor());
				values.put(DbContract.Room.COLUMN_NAME_BOUNDARY_RADIUS, r.getBoundaryRadius());

				// insert new row, returns primary key
				db.insert(DbContract.Room.TABLE_NAME, null, values);
			}
		}
		
		db.close();
	}
	
	private synchronized boolean areRoomsAdded(List<Room> rooms) {
		for (Room r : rooms) {
			if (getRoom(r.getLydiaCode()) == null) {
				return false;
			}
		}
		return true;
	}

	public synchronized Room getRoom(String lydiaCode) {
		Room room = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Room.TABLE_NAME +
				" WHERE " + DbContract.Room.COLUMN_NAME_LYDIA_CODE + " = '" + lydiaCode + "'" +
				" LIMIT 1", null);
		
		if (cursor.moveToFirst()) {
			int id = cursor.getInt(1);
			String nr = cursor.getString(2);
			String name = cursor.getString(3);
			String type = cursor.getString(4);
			int buildingId = cursor.getInt(5);
			int floorId = cursor.getInt(6);
			String geoCenterText = cursor.getString(8);
			int floor = cursor.getInt(9);
			int boundaryRadius = cursor.getInt(10);
			
			String[] geoCenterList = geoCenterText.split(",");
			Coord geoCenter = new Coord(Double.parseDouble(geoCenterList[0]), Double.parseDouble(geoCenterList[1]));

			room = new Room(id, nr, name, type, buildingId, floorId, lydiaCode, geoCenter, floor, boundaryRadius);
		}
		
		db.close();
		
		return room;
	}
	
	public synchronized Room getRoom(int id) {
		Room room = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Room.TABLE_NAME +
				" WHERE " + DbContract.Room._ID + " = " + id +
				" LIMIT 1", null);
		
		if (cursor.moveToFirst()) {
			String nr = cursor.getString(2);
			String name = cursor.getString(3);
			String type = cursor.getString(4);
			int buildingId = cursor.getInt(5);
			int floorId = cursor.getInt(6);
			String lydiaCode = cursor.getString(7);
			String geoCenterText = cursor.getString(8);
			int floor = cursor.getInt(9);
			int boundaryRadius = cursor.getInt(10);
			
			String[] geoCenterList = geoCenterText.split(",");
			Coord geoCenter = new Coord(Double.parseDouble(geoCenterList[0]), Double.parseDouble(geoCenterList[1]));
			
			room = new Room(id, nr, name, type, buildingId, floorId, lydiaCode, geoCenter, floor, boundaryRadius);
		}
		
		db.close();
		
		return room;
	}
	
	public synchronized List<Room> getTimetableRooms(int timetableId) {
		List<Room> rooms = new ArrayList<Room>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.TimetableRoom.COLUMN_NAME_LYDIA_CODE + " FROM " + DbContract.TimetableRoom.TABLE_NAME +
				" WHERE " + DbContract.TimetableRoom.COLUMN_NAME_TIMETABLE_ID + " = " + timetableId, null);
		
		while (cursor.moveToNext()) {
			String lydiaCode = cursor.getString(0);
			rooms.add(getRoom(lydiaCode));
		}
		
		db.close();
		
		return rooms;
	}
	
	/*
	 * STUDENT METHODS
	 */
	
	public synchronized DbStatus addStudent(String uuid) {
		DbStatus status;
		
		if (uuid != null) {
			SQLiteDatabase db = getWritableDatabase();
			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
			values.put(DbContract.Student.COLUMN_NAME_UUID, uuid);
	
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.Student.TABLE_NAME, null, values);
				status = DbStatus.OK;
			} catch (SQLiteConstraintException e) {
				status = DbStatus.ENTRY_EXISTS;
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	public synchronized Student getStudent(int studentId) {
		Student student = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Student.TABLE_NAME +
				" WHERE " + DbContract.Student._ID + " = " + studentId +
				" LIMIT 1", null);
		
		if (cursor.moveToNext()) {			
			int id = cursor.getInt(0);
			String username = cursor.getString(1);
			String email = cursor.getString(2);
			int coins = cursor.getInt(3);
			int taps = cursor.getInt(4);
			String studyProgram = cursor.getString(15);
			String uuid = cursor.getString(16);
			int coinTotal = cursor.getInt(17);
			
			// this is safe because the parsing will never fail due to
			// the fact that the timestamp is automatically added by SQLite
			Date timestamp = null;
			try {
				timestamp = FormatHandler.dateFormat.parse(cursor.getString(18));
			} catch (ParseException e) {}
			
			student = new Student(id, username, email, coins, taps, studyProgram, uuid, coinTotal, timestamp);
		}
		
		db.close();
		
		return student;
	}

	public synchronized void updateCoins(int studentId, int value) {
		Student student = getStudent(studentId);
		int coins = student.getCoins();
		int coinTotal = student.getCoinTotal();
		SQLiteDatabase db = getWritableDatabase();
		
		coins += value;
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Student.COLUMN_NAME_COINS, coins);
		
		// we only update the coin total when the value is greater than 0,
		// meaning the method was called when the student opens a chest
		if (value > 0) {
			values.put(DbContract.Student.COLUMN_NAME_COIN_TOTAL, (coinTotal + value));
		}
		
		db.update(DbContract.Student.TABLE_NAME, values, DbContract.Student._ID + " = " + studentId, null);
		
		db.close();
	}
	
	public synchronized void incrementTaps(int studentId, int taps) {
		int currentTaps = getStudent(studentId).getTaps();
		SQLiteDatabase db = getWritableDatabase();
		
		currentTaps += taps;
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Student.COLUMN_NAME_TAPS, currentTaps);
		
		db.update(DbContract.Student.TABLE_NAME, values, DbContract.Student._ID + " = " + studentId, null);
		
		db.close();
	}
	
	/**
	 * This method isn't really necessary as we can just use
	 * getStudentEfforts to get the same data. However, we also
	 * get a lot of data we don't need in this case.
	 * 
	 * @param studentId
	 * @return
	 */
	public synchronized boolean isStudentEffortUpToDate(int studentId) {
		boolean upToDate = false;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.Student.COLUMN_NAME_EFFORT_UP_TO_DATE + " FROM " +
				DbContract.Student.TABLE_NAME + " WHERE " +
				DbContract.Student._ID + " = " + studentId + " LIMIT 1", null);
		
		if (cursor.moveToNext()) {			
			upToDate = cursor.getShort(0) > 0;  // up-to-date = true if > 0
		}
		
		db.close();
		
		return upToDate;
	}
	
	/**
	 * Invalidates the data used by EffortTotalFragment and makes it refresh them.
	 * @param studentId
	 * @param upToDate
	 */
	public synchronized void setStudentEffortUpToDate(int studentId, boolean upToDate) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Student.COLUMN_NAME_EFFORT_UP_TO_DATE, upToDate);
		
		db.update(DbContract.Student.TABLE_NAME, values, DbContract.Student._ID + " = " + studentId, null);
		
		db.close();
	}
	
	public synchronized void updateStudentEfforts(Student student) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_TOTAL, student.getActivitiesTotal());
		values.put(DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_PASSED, student.getActivitiesPassed());
		values.put(DbContract.Student.COLUMN_NAME_EFFORT_ACTIVITIES_ATTENDED, student.getActivitiesAttended());
		
		AttendanceStats highest = student.getHighestStat();
		if (highest != null && highest.getCourse() != null) {
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_NAME, highest.getCourse().getName());
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_ATTENDED, highest.getAttended());
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_HIGHEST_TOTAL, highest.getTotal());
		}
		
		AttendanceStats lowest = student.getLowestStat();
		if (lowest != null && lowest.getCourse() != null) {
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_NAME, lowest.getCourse().getName());
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_ATTENDED, lowest.getAttended());
			values.put(DbContract.Student.COLUMN_NAME_EFFORT_COURSE_LOWEST_TOTAL, lowest.getTotal());
		}
		
		values.put(DbContract.Student.COLUMN_NAME_EFFORT_UP_TO_DATE, student.isUpToDate());
		
		db.update(DbContract.Student.TABLE_NAME, values, DbContract.Student._ID + " = " + student.getId(), null);
		
		db.close();
	}
	
	public synchronized Student getStudentEfforts(int studentId) {
		Student student = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Student.TABLE_NAME +
				" WHERE " + DbContract.Student._ID + " = " + studentId +
				" LIMIT 1", null);
		
		if (cursor.moveToNext()) {			
			int activitesTotal = cursor.getInt(5);
			int activitesPassed = cursor.getInt(6);
			int activitesAttended = cursor.getInt(7);
			
			String courseName = cursor.getString(8);
			int courseAttended = cursor.getInt(9);
			int courseTotal = cursor.getInt(10);
			double percent = ((double) courseAttended / courseTotal) * 100;
			AttendanceStats highest = new AttendanceStats(new Course(courseName), courseAttended, courseTotal, percent);
		
			courseName = cursor.getString(11);
			courseAttended = cursor.getInt(12);
			courseTotal = cursor.getInt(13);
			percent = ((double) courseAttended / courseTotal) * 100;
			AttendanceStats lowest = new AttendanceStats(new Course(courseName), courseAttended, courseTotal, percent);
			
			boolean upToDate = cursor.getInt(14) > 0;
			
			student = new Student(studentId, activitesTotal, activitesPassed, activitesAttended, highest, lowest, upToDate);
		}
		
		db.close();
		
		return student;
	}
	
	public synchronized void updateStudyProgram(int studentId, String studyProgram) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.Student.COLUMN_NAME_STUDY_PROGRAM, studyProgram);
		
		db.update(DbContract.Student.TABLE_NAME, values, DbContract.Student._ID + " = " + studentId, null);
		
		db.close();
	}
	
	/*
	 * STUDENT COURSES METHODS
	 */
	
	public synchronized DbStatus addCourse(int studentId, String courseCode) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (studentId > 0 && courseCode != null) {
			SQLiteDatabase db = getWritableDatabase();
			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
	
			values.put(DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID, studentId);
			values.put(DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE, courseCode);
	
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.StudentCourse.TABLE_NAME, null, values);
				status = DbStatus.OK;
			} catch (SQLiteConstraintException e) {
				status = DbStatus.ENTRY_EXISTS;
			} finally {
				db.close();
			}
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Checks whether or not a student has already added a course
	 * given the specific course code for that course.
	 * 
	 * @param studentId
	 * @param courseCode
	 * @return
	 */
	public synchronized DbStatus isCourseAdded(int studentId, String courseCode) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (studentId > 0 && courseCode != null) {
			SQLiteDatabase db = getReadableDatabase();
			status = DbStatus.OK;
			
			Cursor cursor = db.rawQuery(
					"SELECT " + DbContract.StudentCourse._ID + " FROM " + DbContract.StudentCourse.TABLE_NAME +
					" WHERE " + DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = " + studentId +
					" AND " + DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + " = '" + courseCode + "'" +
					" LIMIT 1", null);
			
			// if this is true, the course has already been added
			if (cursor.moveToFirst()) {
				status = DbStatus.ENTRY_EXISTS;
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Wipes all traces of the course being added to the app from
	 * the system (including positions and attendances). The only
	 * thing left is the rooms that was associated with timetables.
	 * 
	 * @param studentId
	 * @param courseCode
	 * @return
	 */
	public synchronized DbStatus removeCourse(int studentId, String courseCode) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (studentId > 0 && courseCode != null) {
			
			int success = 0;
			
			List<CourseActivity> activities = getCourseActivities(courseCode);
			for (CourseActivity activity : activities) {
				
				List<Integer> timetableIds = getTimetableIds(activity.getActivityId());
				for (Integer timetableId : timetableIds) {
					
					List<Integer> attendanceIds = getAttendanceIds(studentId, timetableId);
					for (Integer attendanceId : attendanceIds) {
						SQLiteDatabase db = getWritableDatabase();
						
						// delete student positions
						success = db.delete(
								DbContract.StudentPosition.TABLE_NAME,
								DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + " = ? AND " + 
								DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + " = ?",
								new String[]{String.valueOf(studentId), String.valueOf(attendanceId)});
						
						db.close();
					}
					
					SQLiteDatabase db = getWritableDatabase();
					
					// delete attendances
					success = db.delete(
							DbContract.Attendance.TABLE_NAME,
							DbContract.Attendance.COLUMN_NAME_TIMETABLE_ID + " = ? AND " + 
							DbContract.Attendance.COLUMN_NAME_STUDENT_ID + " = ?",
							new String[]{String.valueOf(timetableId), String.valueOf(studentId)});
					
					// delete timetable rooms
					success = db.delete(
							DbContract.TimetableRoom.TABLE_NAME,
							DbContract.TimetableRoom.COLUMN_NAME_TIMETABLE_ID + " = ?",
							new String[]{String.valueOf(timetableId)});
					
					db.close();
				}
				
				SQLiteDatabase db = getWritableDatabase();
				
				// delete timetables
				success = db.delete(
						DbContract.Timetable.TABLE_NAME,
						DbContract.Timetable.COLUMN_NAME_ACTIVITY_ID + " = ?",
						new String[]{activity.getActivityId()});
				
				db.close();
			}
			
			SQLiteDatabase db = getWritableDatabase();
			
			// delete student course efforts
			success = db.delete(
					DbContract.StudentCourseEffort.TABLE_NAME,
					DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID + " = ? AND " +
					DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE + " = ?",
					new String[]{String.valueOf(studentId), courseCode});

			// delete course activities
			success = db.delete(
					DbContract.CourseActivity.TABLE_NAME,
					DbContract.CourseActivity.COLUMN_NAME_COURSE_CODE + " = ?",
					new String[]{courseCode});
			
			// delete student course
			success = db.delete(
					DbContract.StudentCourse.TABLE_NAME,
					DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + " = ? AND " +
					DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = ?",
					new String[]{courseCode, String.valueOf(studentId)});
			
			db.close();
			
			// we do this in case the course that was just removed would've been
			// visible on the effort total fragment
			setStudentEffortUpToDate(studentId, false);
			
			if (success == 1) {
				status = DbStatus.OK;
			} else {
				status = DbStatus.UNKNOWN_ERROR;
			}
		}
		
		return status;
	}
	
	public synchronized List<Course> getStudentCourses(int studentId) {
		List<Course> courses = new ArrayList<Course>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.StudentCourse.TABLE_NAME +
				" WHERE " + DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = " + studentId + "", null);
		
		while (cursor.moveToNext()) {			
			String courseCode = cursor.getString(1);
			courses.add(getCourse(courseCode));
		}
		
		db.close();
		
		return courses;
	}
	
	public synchronized List<String> getStudentCourseCodes(int studentId) {
		List<String> codes = new ArrayList<String>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + " FROM " + DbContract.StudentCourse.TABLE_NAME +
				" WHERE " + DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = " + studentId, null);
		
		while (cursor.moveToNext()) {
			codes.add(cursor.getString(0));
		}
		
		db.close();
		
		return codes;
	}
	
	/*
	 * STUDENT POSITION METHODS
	 */

	public synchronized DbStatus addStudentPosition(StudentPosition pos) {
		DbStatus status;
		
		if (pos != null) {
			SQLiteDatabase db = getWritableDatabase();
			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
	
			values.put(DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID, pos.getStudentId());
			values.put(DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID, pos.getAttendanceId());
			values.put(DbContract.StudentPosition.COLUMN_NAME_LONGITUDE, pos.getLongitude());
			values.put(DbContract.StudentPosition.COLUMN_NAME_LATITUDE, pos.getLatitude());
			values.put(DbContract.StudentPosition.COLUMN_NAME_FLOOR, pos.getFloor());
			values.put(DbContract.StudentPosition.COLUMN_NAME_INSIDE, pos.isInside());
			values.put(DbContract.StudentPosition.COLUMN_NAME_PROVIDER, pos.getProvider().toString());
	
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.StudentPosition.TABLE_NAME, null, values);
				status = DbStatus.OK;
			} catch (SQLException e) {
				status = DbStatus.UNKNOWN_ERROR;
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}

	/**
	 * Gets us all the positions that was recorded for a
	 * student and a given activity (identified with attendanceId).
	 * 
	 * @param studentId
	 * @param attendanceId
	 * @return
	 */
	public synchronized List<StudentPosition> getStudentPositions(int studentId, int attendanceId) {
		List<StudentPosition> positions = new ArrayList<StudentPosition>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.StudentPosition.TABLE_NAME + 
				" WHERE " + DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + " = ?" +
				" AND " + DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + " = ?",
				new String[]{String.valueOf(studentId), String.valueOf(attendanceId)});
		
		while (cursor.moveToNext()) {
			int id = cursor.getInt(0);
			String timestamp = cursor.getString(3);
			String longitude = cursor.getString(4);
			String latitude = cursor.getString(5);
			int floor = cursor.getInt(6);
			boolean inside = cursor.getInt(7) > 0;
			
			positions.add(new StudentPosition(id, studentId, attendanceId, timestamp, longitude, latitude, floor, inside));
		}
		
		db.close();
		
		return positions;
	}
	
	public synchronized int getStudentPositionsInside(int studentId, int attendanceId) {
		int positionsInside = 0;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.StudentPosition._ID + " FROM " + DbContract.StudentPosition.TABLE_NAME + 
				" WHERE " + DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + " = ?" +
				" AND " + DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + " = ?" +
				" AND " + DbContract.StudentPosition.COLUMN_NAME_INSIDE + " = 1", // 1 = true
				new String[]{String.valueOf(studentId), String.valueOf(attendanceId)});
		
		while (cursor.moveToNext()) {
			positionsInside++;
		}
		
		db.close();
		
		return positionsInside;
	}
	
	/**
	 * This method checks whether or not the last position registered for
	 * a given student and activity is considered inside or outside any
	 * of the designated rooms.
	 * 
	 * @param studentId
	 * @param attendanceId
	 * @return
	 */
	public synchronized boolean isLastStudentPositionInside(int studentId, int attendanceId) {
		boolean inside = false;
		SQLiteDatabase db = getReadableDatabase();

		Cursor cursor = db.rawQuery(
				"SELECT " + DbContract.StudentPosition.COLUMN_NAME_INSIDE + " FROM " + DbContract.StudentPosition.TABLE_NAME + 
				" WHERE " + DbContract.StudentPosition.COLUMN_NAME_STUDENT_ID + " = ?" +
				" AND " + DbContract.StudentPosition.COLUMN_NAME_ATTENDANCE_ID + " = ?" +
				" AND " + DbContract.StudentPosition._ID + " = (SELECT MAX(" + DbContract.StudentPosition._ID + ") FROM " +  DbContract.StudentPosition.TABLE_NAME + ") LIMIT 1",
				new String[]{String.valueOf(studentId), String.valueOf(attendanceId)});
		
		if (cursor.moveToFirst()) {
			inside = cursor.getInt(0) > 0; // 1 = true
		}
		
		db.close();
		
		return inside;
	}
	
	/*
	 * STUDENT ITEM METHODS
	 */

	public synchronized DbStatus addLoot(int studentId, Item loot) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (studentId > 0 && loot != null) {
			SQLiteDatabase db = getWritableDatabase();

			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
			values.put(DbContract.StudentItem.COLUMN_NAME_TYPE, loot.getType().getId());
			values.put(DbContract.StudentItem.COLUMN_NAME_NAME, loot.getName());
			
			// only chests have a value as of now, but maybe CONSUMABLES will
			// get a value later on for the purpose of selling them?
			int value = -1;
			int difficulty = -1;
			if (loot instanceof Chest) {
				Chest c = (Chest) loot;
				value = c.getValue();
				difficulty = c.getDifficulty();
			}

			values.put(DbContract.StudentItem.COLUMN_NAME_VALUE, value);
			values.put(DbContract.StudentItem.COLUMN_NAME_USED, loot.isUsed());
			values.put(DbContract.StudentItem.COLUMN_NAME_DIFFICULTY, difficulty);
			values.put(DbContract.StudentItem.COLUMN_NAME_STUDENT_ID, studentId);
			
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.StudentItem.TABLE_NAME, null, values);
				status = DbStatus.OK;
			} catch (SQLException e) {
				status = DbStatus.UNKNOWN_ERROR;
			}

			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Adds items to the student's inventory when buying from the shop.
	 * 
	 * @param studentId
	 * @param item
	 * @return
	 */
	public synchronized DbStatus addItem(int studentId, ShopItem item) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (item != null) {
			SQLiteDatabase db = getWritableDatabase();
			
			for (int i = 0; i < item.getAmount(); i++) {
				// create new map of values, where column names are the keys
				ContentValues values = new ContentValues();
				values.put(DbContract.StudentItem.COLUMN_NAME_TYPE, item.getType().getId());
				values.put(DbContract.StudentItem.COLUMN_NAME_NAME, item.getName());
				values.put(DbContract.StudentItem.COLUMN_NAME_VALUE, item.getPrice()); // how much the student paid
				values.put(DbContract.StudentItem.COLUMN_NAME_USED, item.isUsed());
				values.put(DbContract.StudentItem.COLUMN_NAME_DIFFICULTY, -1);
				values.put(DbContract.StudentItem.COLUMN_NAME_STUDENT_ID, studentId);
				
				// insert new row, insert returns primary key
				try {
					db.insertOrThrow(DbContract.StudentItem.TABLE_NAME, null, values);
					status = DbStatus.OK;
				} catch (SQLException e) {
					status = DbStatus.UNKNOWN_ERROR;
				}
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Gets loot that is not used based on the given ItemType.
	 * 
	 * @param studentId
	 * @param type
	 * @return
	 */
	public synchronized List<Item> getLoot(int studentId, ItemType type) {
		List<Item> loot = new ArrayList<Item>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.StudentItem.TABLE_NAME + 
				" WHERE " + DbContract.StudentItem.COLUMN_NAME_STUDENT_ID + " = ?" +
				" AND " + DbContract.StudentItem.COLUMN_NAME_TYPE + " = ?" +
				" AND " + DbContract.StudentItem.COLUMN_NAME_USED + " = 0",
				new String[]{String.valueOf(studentId), String.valueOf(type.getId())});
		
		while (cursor.moveToNext()) {
			// not used as of now 
			int id = cursor.getInt(0);
			
			//ItemType it = ItemType.getItemType(cursor.getInt(1));
			
			// not used as of now
			//String name = cursor.getString(2);
			int value = cursor.getInt(3);
			boolean used = cursor.getInt(4) > 0; // used = true if > 0
			int difficulty = cursor.getInt(5);
			
			// not sure if we want to display the date
			// the student got the item yet
			//String timestamp = cursor.getString(6);
			
			Item item = null;
			switch (type) {
				case CHEST:
					item = new Chest(id, value, difficulty, used);
					break;
				case ICE_AXE:
					item = new IceAxe(id);
					break;
				default:
					break;
			}
			
			loot.add(item);
		}
		
		db.close();
		
		return loot;
	}
	
	public synchronized List<IceAxe> getIceAxes(int studentId) {
		List<IceAxe> loot = new ArrayList<IceAxe>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.StudentItem.TABLE_NAME + 
				" WHERE " + DbContract.StudentItem.COLUMN_NAME_STUDENT_ID + " = ?" +
				" AND " + DbContract.StudentItem.COLUMN_NAME_TYPE + " = ?" +
				" AND " + DbContract.StudentItem.COLUMN_NAME_USED + " = 0",
				new String[]{String.valueOf(studentId), String.valueOf(ItemType.ICE_AXE.getId())}
		);
		
		while (cursor.moveToNext()) {
			// not used as of now 
			int id = cursor.getInt(0);
			
			// not used as of now
			//String name = cursor.getString(2);
			
			// not sure if we want to display the date
			// the student got the item yet
			//String timestamp = cursor.getString(6);
			
			loot.add(new IceAxe(id));
		}
		
		db.close();
		
		return loot;
	}
	
	public synchronized void updateLoot(Item item) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.StudentItem.COLUMN_NAME_USED, item.isUsed());
		
		db.update(DbContract.StudentItem.TABLE_NAME, values, DbContract.StudentItem._ID + " = " + item.getId(), null);
		
		db.close();
	}
	
	/*
	 * ACHIEVEMENT METHODS
	 */

	public synchronized int getNumberOfChests(int studentId) {
		int numberOfChests = 0;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT Count(" + DbContract.StudentItem.COLUMN_NAME_TYPE + ") FROM " + DbContract.StudentItem.TABLE_NAME +
				" WHERE " + DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.StudentItem.COLUMN_NAME_TYPE + " = " + ItemType.CHEST.getId(), null);
		
		if (cursor.moveToFirst()) {
			numberOfChests = cursor.getInt(0);
		}
		
		db.close();
		
		return numberOfChests;
	}

	/**
	 * Gets all the information about the achievements including type and timestamp when
	 * they were added. Ordered by the timestamp when the student got the achievement.
	 * 
	 * @param studentId
	 * @return
	 */
	public synchronized List<Achievement> getStudentAchievements(int studentId) {
		List<Achievement> achievements = new ArrayList<Achievement>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT ach.*, stud." + DbContract.StudentAchievement.COLUMN_NAME_TIMESTAMP +
				" FROM " + DbContract.StudentAchievement.TABLE_NAME +
				" AS stud JOIN " + DbContract.Achievement.TABLE_NAME +
				" AS ach ON stud." + DbContract.StudentAchievement.COLUMN_NAME_ACHIEVEMENT_ID +
				" = ach." + DbContract.Achievement.COLUMN_NAME_ID + "" +
				" WHERE stud." + DbContract.StudentAchievement.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" ORDER BY stud." + DbContract.StudentAchievement.COLUMN_NAME_TIMESTAMP + " DESC", null);
		
		while (cursor.moveToNext()) {
			int id = cursor.getInt(0);
			int category = cursor.getInt(1);
			AchievementType type = AchievementHandler.getAchievementTypeFromId(id);
			String name = cursor.getString(2);
			String description = cursor.getString(3);
			int value = cursor.getInt(4);
			
			// this is safe because the parsing will never fail
			// due to the fact that all achievements will have a timestamp
			Date timestamp = null;
			try {
				timestamp = FormatHandler.dateFormat.parse(cursor.getString(5));
			} catch (ParseException e) {}
			
			achievements.add(new Achievement(id, category, type, name, description, value, timestamp));
		}
		
		db.close();
		
		return achievements;
	}
	
	/**
	 * Gets the full achievements given a list of only achievement types.
	 * 
	 * @param types
	 * @return
	 */
	public synchronized List<Achievement> getAchievements(List<AchievementType> types) {
		List<Achievement> achievements = new ArrayList<Achievement>();
		
		for (AchievementType type : types) {
			Achievement a = getAchievement(type);
			if (a != null) {
				achievements.add(a);
			}
		}
		
		return achievements;
	}
	
	/**
	 * Gets the full achievements given a list of only achievement types.
	 * 
	 * @param types
	 * @return
	 */
	public synchronized List<Achievement> getAchievements(AchievementType[] types) {
		List<Achievement> achievements = new ArrayList<Achievement>();
		
		for (AchievementType type : types) {
			Achievement a = getAchievement(type);
			if (a != null) {
				achievements.add(a);
			}
		}
		
		return achievements;
	}
	
	/**
	 * Gets a single achievement given the achievement type
	 * (if it exists in the database). Returns null otherwise.
	 * 
	 * @param type
	 * @return
	 */
	private synchronized Achievement getAchievement(AchievementType type) {
		Achievement achievement = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.Achievement.TABLE_NAME +
				" WHERE " + DbContract.Achievement.COLUMN_NAME_ID + " = " + type.getId() +
				" LIMIT 1", null);
		
		if (cursor.moveToFirst()) {
			int id = cursor.getInt(0);
			int category = cursor.getInt(1);
			String name = cursor.getString(2);
			String description = cursor.getString(3);
			int value = cursor.getInt(4);

			achievement = new Achievement(id, category, name, description, value);
		}

		db.close();
		
		return achievement;
	}
	
	/**
	 * Fills the database with achievements that
	 * correspond to the enum type (AchievementType).
	 * 
	 * @param achievements
	 * @return
	 */
	public synchronized DbStatus addAchievements(List<Achievement> achievements) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (!achievements.isEmpty()) {
			SQLiteDatabase db = getWritableDatabase();
			
			for (Achievement a : achievements) {
				// create new map of values, where column names are the keys
				ContentValues values = new ContentValues();
				values.put(DbContract.Achievement.COLUMN_NAME_ID, a.getId());
				values.put(DbContract.Achievement.COLUMN_NAME_CATEGORY, a.getCategory());
				values.put(DbContract.Achievement.COLUMN_NAME_NAME, a.getName());
				values.put(DbContract.Achievement.COLUMN_NAME_DESCRIPTION, a.getDescription());
				values.put(DbContract.Achievement.COLUMN_NAME_VALUE, a.getValue());
				
				// insert new row, insert returns primary key
				try {
					db.insertOrThrow(DbContract.Achievement.TABLE_NAME, null, values);
					status = DbStatus.OK;
				} catch (SQLException e) {
					status = DbStatus.UNKNOWN_ERROR;
				}
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}

	public synchronized void addStudentAchievements(int studentId, List<Achievement> achievements) {		
		if (!achievements.isEmpty()) {
			SQLiteDatabase db = getWritableDatabase();
			
			for (Achievement a : achievements) {
				// create new map of values, where column names are the keys
				ContentValues values = new ContentValues();
				values.put(DbContract.StudentAchievement.COLUMN_NAME_STUDENT_ID, studentId);
				values.put(DbContract.StudentAchievement.COLUMN_NAME_ACHIEVEMENT_ID, a.getId());
				
				// insert new row, insert returns primary key
				try {
					db.insertOrThrow(DbContract.StudentAchievement.TABLE_NAME, null, values);
				} catch (SQLException e) {}
			}
			
			db.close();
		}
	}
	
	/*
	 * SHOP ITEM METHODS
	 */
	
	/**
	 * Adds a new item to the shop.
	 * 
	 * @param studentId
	 * @param item
	 * @return
	 */
	public synchronized DbStatus addShopItem(ShopItem item) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (item != null) {
			SQLiteDatabase db = getWritableDatabase();
			
			// create new map of values, where column names are the keys
			ContentValues values = new ContentValues();
			values.put(DbContract.ShopItem.COLUMN_NAME_TYPE, item.getType().getId());
			values.put(DbContract.ShopItem.COLUMN_NAME_NAME, item.getName());
			values.put(DbContract.ShopItem.COLUMN_NAME_DESCRIPTION, item.getDescription());
			values.put(DbContract.ShopItem.COLUMN_NAME_PRICE, item.getPrice());
			values.put(DbContract.ShopItem.COLUMN_NAME_AMOUNT, item.getAmount());
			values.put(DbContract.ShopItem.COLUMN_NAME_REQUIREMENT, item.getRequirement());
			
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.ShopItem.TABLE_NAME, null, values);
				status = DbStatus.OK;
			} catch (SQLException e) {
				status = DbStatus.UNKNOWN_ERROR;
			}
			
			db.close();
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Fills the database with shop items that
	 * correspond to the enum type (ItemType).
	 * 
	 * @param achievements
	 * @return
	 */
	public synchronized DbStatus addShopItems(List<ShopItem> items) {
		DbStatus status = DbStatus.NOT_INITIALIZED;
		
		if (!items.isEmpty()) {
			
			// TODO: use prepared statements/batch here instead?
			for (ShopItem item : items) {
				addShopItem(item);
			}
			
			status = DbStatus.OK;
		} else {
			status = DbStatus.INVALID_DATA;
		}
		
		return status;
	}
	
	/**
	 * Gets all the items for the shop.
	 * 
	 * @return
	 */
	public synchronized List<ShopItem> getShopItems() {
		List<ShopItem> items = new ArrayList<ShopItem>();
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery("SELECT * FROM " + DbContract.ShopItem.TABLE_NAME, null);
		
		while (cursor.moveToNext()) {
			// not used as of now 
			//int id = cursor.getInt(0);

			ItemType type = ItemType.getItemType(cursor.getInt(1));
			String name = cursor.getString(2);
			String description = cursor.getString(3);
			int price = cursor.getInt(4);
			int amount = cursor.getInt(5);
			int requirement = cursor.getInt(6);	
			
			items.add(new ShopItem(type, name, description, price, amount, requirement));
		}
		
		db.close();
		
		return items;
	}
	
	/*
	 * STUDENT COURSE EFFORT
	 */
	
	public synchronized StudentCourseEffort getStudentCourseEffort(int studentId, String courseCode) {
		StudentCourseEffort effort = null;
		SQLiteDatabase db = getReadableDatabase();
		
		Cursor cursor = db.rawQuery(
				"SELECT * FROM " + DbContract.StudentCourseEffort.TABLE_NAME +
				" WHERE " + DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID + " = " + studentId +
				" AND " + DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE + " = '" + courseCode + "'" +
				" LIMIT 1", null);
		
		if (cursor.moveToFirst()) {
			int activitiesTotal = cursor.getInt(3);
			int activitiesPassed = cursor.getInt(4);
			int activitiesAttended = cursor.getInt(5);
			int lecturesPassed = cursor.getInt(6);
			int lecturesAttended = cursor.getInt(7);
			int lecturesFull = cursor.getInt(8);
			int exercisesPassed = cursor.getInt(9);
			int exercisesAttended = cursor.getInt(10);
			boolean upToDate = cursor.getInt(11) > 0; // 1 = true
			
			effort = new StudentCourseEffort(
					studentId, courseCode, activitiesTotal, activitiesPassed, activitiesAttended,
					lecturesPassed, lecturesAttended, lecturesFull, exercisesPassed, exercisesAttended, upToDate);
		}
		
		db.close();
		
		return effort;
	}
	
	public synchronized void updateStudentCourseEffort(StudentCourseEffort effort) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_TOTAL, effort.getActivitiesTotal());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_ATTENDED, effort.getActivitiesAttended());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_PASSED, effort.getActivitiesPassed());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_PASSED, effort.getLecturesPassed());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_ATTENDED, effort.getLecturesAtteneded());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_FULL, effort.getLecturesFull());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_PASSED, effort.getExercisesPassed());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_ATTENDED, effort.getExercisesAttended());
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_UP_TO_DATE, effort.isUpToDate());
		
		db.update(DbContract.StudentCourseEffort.TABLE_NAME, values,
				DbContract.StudentCourse.COLUMN_NAME_STUDENT_ID + " = " + effort.getStudentId() + " AND " + 
				DbContract.StudentCourse.COLUMN_NAME_COURSE_CODE + " = '" + effort.getCourseCode() + "'", null);
		
		db.close();
	}
	
	public synchronized void addStudentCourseEffort(StudentCourseEffort effort) {		
		if (effort != null) {
			SQLiteDatabase db = getWritableDatabase();

			ContentValues values = new ContentValues();
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID, effort.getStudentId());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE, effort.getCourseCode());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_TOTAL, effort.getActivitiesTotal());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_ATTENDED, effort.getActivitiesAttended());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_ACTIVITIES_PASSED, effort.getActivitiesPassed());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_PASSED, effort.getLecturesPassed());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_ATTENDED, effort.getLecturesAtteneded());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_LECTURES_FULL, effort.getLecturesFull());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_PASSED, effort.getExercisesPassed());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_EXERCISES_ATTENDED, effort.getExercisesAttended());
			values.put(DbContract.StudentCourseEffort.COLUMN_NAME_UP_TO_DATE, effort.isUpToDate());
			
			// insert new row, insert returns primary key
			try {
				db.insertOrThrow(DbContract.StudentCourseEffort.TABLE_NAME, null, values);
			} catch (SQLException e) {
				e.printStackTrace();
			}
			
			db.close();
		}
	}
	
	/**
	 * Invalidates the data used by EffortCourseFragment and makes it refresh them.
	 * This is on a course to course basis, and the data is only refreshed when the
	 * user tries to access the said fragment again.
	 * 
	 * Invalidating a single course automatically invalidates the data used by
	 * EffortTotalFragment. 
	 * 
	 * @param studentId
	 * @param courseCode
	 * @param upToDate
	 */
	public synchronized void setStudentCourseEffortUpToDate(int studentId, String courseCode, boolean upToDate) {
		setStudentEffortUpToDate(studentId, upToDate);
		
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues values = new ContentValues();
		values.put(DbContract.StudentCourseEffort.COLUMN_NAME_UP_TO_DATE, upToDate);
		
		db.update(
				DbContract.StudentCourseEffort.TABLE_NAME, values,
				DbContract.StudentCourseEffort.COLUMN_NAME_STUDENT_ID + " = " + studentId + " AND " +
				DbContract.StudentCourseEffort.COLUMN_NAME_COURSE_CODE + " = '" + courseCode + "'", null);
		
		db.close();
	}

}