package no.skarasoftwaredevelopment.schooltracker.gui;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper.DbStatus;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.CourseAdapter;
import no.skarasoftwaredevelopment.schooltracker.handlers.ToastHandler;
import no.skarasoftwaredevelopment.schooltracker.parsers.ScheduleParser;
import no.skarasoftwaredevelopment.schooltracker.pojo.Course;
import no.skarasoftwaredevelopment.schooltracker.pojo.Schedule;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.services.SchedulingService;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TabHost;
import android.widget.TextView;

public class AddCourseFragment extends Fragment {

	public static final String TAG = "AddCourseFragment";

	private static final int WAIT_TIME = 500;

	private CourseAdapter adapter;
	private Handler handler;
	private Runnable filterTask;
	private Course newCourse;
	private Activity activity;
	private InputMethodManager imm;
	private DbHelper db;
	private Student student;
	private boolean switchTabs;
    private boolean paused;
	
	private EditText query;
	private ListView list;
	private Button button;
	private TabHost tabHost;
	private TextView empty;

    public AddCourseFragment() {
        // Empty constructor required for fragment subclasses
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) { 
        View rootView = inflater.inflate(R.layout.fragment_add_course, container, false);
        
        activity = getActivity();
        paused = false;
        db = new DbHelper(activity);
        
        query = (EditText) rootView.findViewById(R.id.fragment_add_course_queryEditText);
        empty = (TextView) rootView.findViewById(R.id.fragment_add_course_emptyTextView);
        
        student = db.getStudent(1); // TODO: get studentId dynamically
        if (student == null || student.getStudyProgram() == null) {
        	query.setEnabled(false);
        	empty.setText(R.string.AddCoursesFragment_text_missing_study_program);
        } else {
            imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
            handler = new Handler();
            filterTask = new FilterTask();

            tabHost = (TabHost) activity.findViewById(android.R.id.tabhost);
        
	        query.addTextChangedListener(new TextWatcher() {
				@Override
				public void afterTextChanged(Editable arg0) {
				}
	
				@Override
				public void beforeTextChanged(CharSequence arg0, int arg1, int arg2, int arg3) {
				}
	
				@Override
				public void onTextChanged(CharSequence arg0, int arg1, int arg2, int arg3) {
					handler.removeCallbacks(filterTask); 
					handler.postDelayed(filterTask, WAIT_TIME);
				}
			});
	        
			list = (ListView) rootView.findViewById(R.id.fragment_add_course_listListView);
			list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> l, View v, int position, long id) {
					if (!button.isEnabled()) {
						button.setEnabled(true);
					}
					
					newCourse = adapter.getItem(position);
					
					// hide the virtual keyboard when the student picks a course from the list
					imm.hideSoftInputFromWindow(list.getWindowToken(), 0);
					list.requestFocusFromTouch(); // we need this to maintain the highlighted list item
				}
			});
	
	        button = (Button) rootView.findViewById(R.id.fragment_add_course_addButton);
	        button.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					// prevents the student from adding the same course multiple times
					button.setEnabled(false);
					
					ConnectivityManager conManager = (ConnectivityManager) activity.getSystemService(Context.CONNECTIVITY_SERVICE);
	        	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();

	        	    if (networkInfo != null && networkInfo.isConnected()) {
						DbStatus status = db.isCourseAdded(1,  newCourse.getCode()); // TODO: get studentId dynamically
						
						if (status == DbStatus.OK) {	
		        	    	// cache complete schedule for the new course
		        	    	Log.e(TAG, "adding a new course, start caching the schedule..");
		        	    	
							// start downloading the schedule
		        	    	new ParseTask().execute();
						} else if (status == DbStatus.ENTRY_EXISTS) {
							ToastHandler.shortMsg(activity, R.string.AddCoursesFragment_toast_exists_hint);
						} else if (status == DbStatus.INVALID_DATA) {
							ToastHandler.shortMsg(activity, R.string.AddCoursesFragment_toast_invalid_data_hint);
						}
					} else {
						ToastHandler.shortMsg(activity, R.string.AddCoursesFragment_toast_no_internet_connection);
					}
				}
			});
        }

		return rootView;
    }
    
    private class FilterTask implements Runnable {
	    @Override
	    public void run() {
			String str = query.getText().toString();

			if (str != null && !str.equals("")) {
				empty.setVisibility(View.INVISIBLE);
				DbHelper db = new DbHelper(activity);
				adapter = new CourseAdapter(activity, db.getCourses(str));
				list.setAdapter(adapter);

				if (adapter.getCount() == 0) {
					empty.setText(R.string.AddCoursesFragment_text_no_results);
					empty.setVisibility(View.VISIBLE);
				} else {
					empty.setText(R.string.AddCoursesFragment_text_add_hint);
				}
			} else {
				if (adapter != null) {
					adapter.clear();
				}
				newCourse = null;
				button.setEnabled(false);

				// show explanation if the list is empty (no search query)
				// we need to do it this way instead of using the built in functions
				// in the ListFragment due to the menu drawer (overrides the listener)
				empty.setText(R.string.AddCoursesFragment_text_add_hint);
				empty.setVisibility(View.VISIBLE);
			}

			Log.e("AddCourseFragment", "searching for '" + str + "'"); //
	    }
	};

    private class ParseTask extends AsyncTask<String, String, String> implements Runnable {

        private AlertDialog dialog;
        private boolean hasActivities;
        private DbStatus status;
        
        @SuppressLint("InflateParams")
		@Override
		protected void onPreExecute() {
			View v = activity.getLayoutInflater().inflate(R.layout.dialog_loader, null);
			dialog = new AlertDialog.Builder(activity)
					.setView(v)
					.setCancelable(false) // TODO: it should be possible to cancel this
					.setTitle(R.string.AddCoursesFragment_dialog_downloading_schedule).create();
			dialog.show();
			
			// we use this to check if the added course has any activities this semester
	    	hasActivities = false;
		}
		
		@Override
		protected String doInBackground(String... input) {			
			Schedule schedule = ScheduleParser.parseSchedule(newCourse.getCode());
			
			if (schedule != null && !schedule.getActivities().isEmpty()) {
				hasActivities = true;
				db.addCourse(1, newCourse.getCode()); // TODO: get studentId dynamically
				status = db.cacheSchedule(schedule);
				
				if (status == DbStatus.OK) {
					db.setStudentEffortUpToDate(1, false); // TODO: get studentId dynamically
			    	SchedulingService.scheduleAlarms(activity);
				}
			} else {
				status = DbStatus.NO_ACTIVITIES;
			}
			
			return null;
		}
		
		@Override
		protected void onProgressUpdate(String... values) {
		}
	
		@Override
		protected void onPostExecute(String results) {
			dialog.cancel();
			
			if (hasActivities) {
				if (status == DbStatus.OK) {
					list.setAdapter(adapter);
					ToastHandler.longMsg(activity, R.string.AddCoursesFragment_toast_downloaded);

			    	// if the app loses focus (the student goes to the home screen
					// during the download of a course), we need to check for this
			    	// before trying to update the GUI (switching tabs)
					if (!paused) { 
						// we disable the button again because the user added a course
						// and reset the textview so it's clear if the student returns
						tabHost.setCurrentTab(1);
					} else {
						switchTabs = true;
					}
					
					query.setText("");
				} else if (status == DbStatus.MISSING_ROOMS) {
					ToastHandler.longMsg(activity, R.string.AddCoursesFragment_toast_missing_rooms);
				}
			} else if (status == DbStatus.NO_ACTIVITIES) {
				ToastHandler.longMsg(activity, R.string.AddCoursesFragment_toast_no_acitivities);
			} else {
				ToastHandler.longMsg(activity, R.string.AddCoursesFragment_toast_no_course_data);
			}
		}

		@Override
		public void run() {
		}

    }
    
    @Override
    public void onResume() {
        super.onResume();
        
        if (switchTabs && tabHost != null) {
    		switchTabs = false;
        	tabHost.setCurrentTab(1);
    	}
    }
    
    @Override
    public void onPause() {
    	paused = true;
    	super.onPause();
    }

}