package no.skarasoftwaredevelopment.schooltracker.gui;

import java.util.List;
import java.util.Random;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.NotificationHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.SoundHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Chest;
import no.skarasoftwaredevelopment.schooltracker.pojo.IceAxe;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class ChestFragment extends Fragment {
	
	public static final String TAG = "ChestFragment";

    private MainActivity activity;
    private int[][] chestImages;
    private int currentDifficulty;
    private boolean axeEnabled;
    private boolean chestOpened;
    private int tapCounter;
    private int chestStyle;
    private Chest chest;
    
    private TextView chestAlreadyOpened;
    private RelativeLayout layoutContainer;
    private ImageView loot;
    private ImageView iceAxe;
    private TextView chestDescription;
    private TextView openChest;
    private LinearLayout lootContainer;
    private TextView amountOfIceAxes;
    private TextView lootContent;
    
    public ChestFragment() {
    	chestImages = new int[2][10];
    	
    	// chest style: stained glass
    	chestImages[0][0] = R.drawable.chest_ice_stained_glass_01;
    	chestImages[0][1] = R.drawable.chest_ice_stained_glass_02;
    	chestImages[0][2] = R.drawable.chest_ice_stained_glass_03;
    	chestImages[0][3] = R.drawable.chest_ice_stained_glass_04;
    	chestImages[0][4] = R.drawable.chest_ice_stained_glass_05;
    	chestImages[0][5] = R.drawable.chest_ice_stained_glass_06;
    	chestImages[0][6] = R.drawable.chest_ice_stained_glass_07;
    	chestImages[0][7] = R.drawable.chest_ice_stained_glass_08;
    	chestImages[0][8] = R.drawable.chest_ice_stained_glass_09;
    	chestImages[0][9] = R.drawable.chest_ice_stained_glass_10;
    	
    	// chest style: mosaic
    	chestImages[1][0] = R.drawable.chest_ice_mosaic_01;
    	chestImages[1][1] = R.drawable.chest_ice_mosaic_02;
    	chestImages[1][2] = R.drawable.chest_ice_mosaic_03;
    	chestImages[1][3] = R.drawable.chest_ice_mosaic_04;
    	chestImages[1][4] = R.drawable.chest_ice_mosaic_05;
    	chestImages[1][5] = R.drawable.chest_ice_mosaic_06;
    	chestImages[1][6] = R.drawable.chest_ice_mosaic_07;
    	chestImages[1][7] = R.drawable.chest_ice_mosaic_08;
    	chestImages[1][8] = R.drawable.chest_ice_mosaic_09;
    	chestImages[1][9] = R.drawable.chest_ice_mosaic_10;
    	
    	tapCounter = 0;
    	axeEnabled = false;
    	chestOpened = false;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View rootView = inflater.inflate(R.layout.fragment_chest, container, false);        
	    
        activity = (MainActivity) getActivity();
	    activity.setDrawerIndicatorEnabled(false);
        
        chest = (Chest) getArguments().getSerializable(InventoryFragment.ARGS_ITEM);
        currentDifficulty = chest.getDifficulty();
        
        // we need to do this in order to keep the name if the user
        // opens a notification and then hits the back button
        activity.setTitle(chest.getName());
        
        final DbHelper db = new DbHelper(activity);
        final List<IceAxe> axes = db.getIceAxes(1); // TODO: get studentId dynamically
        
        chestAlreadyOpened = (TextView) rootView.findViewById(R.id.fragment_item_chestOpenedTextView); 
        layoutContainer = (RelativeLayout) rootView.findViewById(R.id.fragment_item_layoutContainerRelativeLayout);
        loot = (ImageView) rootView.findViewById(R.id.fragment_item_lootImageView);
        iceAxe = (ImageView) rootView.findViewById(R.id.fragment_item_icePickImageView);
        chestDescription = (TextView) rootView.findViewById(R.id.fragment_item_chestDescriptionTextView);
        openChest = (TextView) rootView.findViewById(R.id.fragment_item_openChestTextView);
        lootContainer = (LinearLayout) rootView.findViewById(R.id.fragment_item_lootContainerLinearLayout);
        amountOfIceAxes = (TextView) rootView.findViewById(R.id.fragment_item_iceAxesAmountTextView);
        lootContent = (TextView) rootView.findViewById(R.id.fragment_item_lootContentTextView);
        
        // decides what style of chest we want to display
    	Random rnd = new Random();
    	chestStyle = rnd.nextInt(chestImages.length);
        setLootImage(chestImages[chestStyle][currentDifficulty]);
        
        loot.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {					
				if (!chestOpened) {
					tapCounter++;
					
					if (axeEnabled && currentDifficulty >= 0) {
						currentDifficulty = -1;
					} else {
						currentDifficulty--;
					}

					if (currentDifficulty >= 0) {
						SoundHandler.playSound(activity, R.raw.hit);
						setLootImage(chestImages[chestStyle][currentDifficulty]);
					} else if (currentDifficulty == -1) {
						SoundHandler.playSound(activity, R.raw.destroy);
						setLootImage(R.drawable.chest);
						chestDescription.setVisibility(View.INVISIBLE);
						openChest.setVisibility(View.VISIBLE);
						
						if (axeEnabled) {
							// remove the first axe in the list
							IceAxe axe = axes.get(0);
							axe.setUsed(axeEnabled);
							
							db.updateLoot(axe);
							
							axes.remove(0);
							if (axes.size() >= 0) {
								amountOfIceAxes.setText(axes.size() + "x");
							}
						}
						
						disableIcePick();
					} else {
						openChest.setVisibility(View.INVISIBLE);
						lootContainer.setVisibility(View.VISIBLE);
						
						int coins = chest.getValue();
						if (coins > 0) {	
							SoundHandler.playSound(activity, R.raw.win);
							setLootImage(R.drawable.chest_win);
							lootContent.setText(getResources().getQuantityString(R.plurals.ChestFragment_text_coins, coins, coins));
							db.updateCoins(1, coins); // TODO: get studentId dynamically
						} else {
							SoundHandler.playSound(activity, R.raw.empty);
							setLootImage(R.drawable.chest_empty);
							lootContent.setText(getResources().getString(R.string.ChestFragment_text_chest_empty));
						}

						chestOpened = true;
						chest.setUsed(chestOpened);
						db.updateLoot(chest);
						db.incrementTaps(1, tapCounter); // TODO: get studentId dynamically 
						
						// check for and add new achievements,
						// and then notify the user with a notification
						List<Achievement> achievements = AchievementHandler.checkAchievements(activity, 1); // TODO: get studentId dynamically 
						
						if (achievements.size() > 0) {
							db.addStudentAchievements(1, achievements); // TODO: dynamically find studentId
							NotificationHandler.notifyAchievement(activity, achievements.size());
						}
					}
				}
			}
		});
        
        if (axes.size() > 0) {        	
	        iceAxe.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (!axeEnabled) {
						iceAxe.setBackgroundResource(R.drawable.background_attending);
					} else {
						iceAxe.setBackgroundResource(R.drawable.background_normal);
					}
					
					axeEnabled = !axeEnabled;
				}
			});
        } else {
        	disableIcePick();
        }
        amountOfIceAxes.setText(axes.size() + "x");
        
        return rootView;
    }
    
    private void setLootImage(int imageResource) {
    	loot.setImageResource(imageResource);
    }
    
    private void disableIcePick() {
		iceAxe.setBackgroundResource(R.drawable.background_disabled);
		iceAxe.setImageResource(R.drawable.ice_axe_disabled);
		iceAxe.setClickable(false);
		iceAxe.setEnabled(false);
    }
    
    @Override
    public void onStop() {
    	super.onStop();
    	
    	// we need to do this in case the user swipes left to show the menu drawer
    	// doing so will use the correct action bar indicator for the menu
    	activity.setDrawerIndicatorEnabled(true);
    }
    
    @Override
    public void onResume() {
    	super.onResume();
    	
    	// if the student opens a notification after having opened a chest,
    	// we hide all the content and display a text explaining this
    	if (chest.isUsed()) {
            chestAlreadyOpened.setVisibility(View.VISIBLE); 
            layoutContainer.setVisibility(View.GONE);
    	}
    	
    	// due to the life cycle of fragments, onResume will get called whenever the
    	// app is running in the background, meaning that we have to disable the
    	// drawer indicator in the action bar again when this fragment gets recreated
    	activity.setDrawerIndicatorEnabled(false);
    }

}