package no.skarasoftwaredevelopment.schooltracker.gui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.NavigationDrawerAdapter;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler.AchievementType;
import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.NotificationHandler;
import no.skarasoftwaredevelopment.schooltracker.parsers.CourseParser;
import no.skarasoftwaredevelopment.schooltracker.parsers.RoomParser;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Course;
import no.skarasoftwaredevelopment.schooltracker.pojo.IceAxe;
import no.skarasoftwaredevelopment.schooltracker.pojo.Item.ItemType;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.ShopItem;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.services.SchedulingService;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.content.res.Configuration;
import android.media.AudioManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.widget.DrawerLayout;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.ListView;

public class MainActivity extends FragmentActivity {
    
	public static final String TAG = "MainActivity";
	
	private Activity activity;
    private String[] menuTitles;
    private CharSequence drawerTitle;
    private CharSequence title;
    private InputMethodManager imm;
    private FragmentManager fragmentManager;
    
    private DrawerLayout drawerLayout;
    private ActionBarDrawerToggle drawerToggle;
    private LinearLayout drawerLinear;
    private ListView drawerList;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		activity = this;
		fragmentManager = getSupportFragmentManager();
		
        // parse JSON and fill the databases, after this has been done
        // we run the AlarmReceiver to check if there should be any tasks
        // running now
        new ParseTask().execute();
		
		menuTitles = getResources().getStringArray(R.array.MainActivity_menu_titles);
        drawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawerLinear = (LinearLayout) findViewById(R.id.left_drawer);
        drawerList = (ListView) findViewById(R.id.left_drawer_list);

        drawerList.setAdapter(new NavigationDrawerAdapter(this, menuTitles));
        drawerList.setOnItemClickListener(new DrawerItemClickListener());
        
        title = drawerTitle = getTitle();
        
        drawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawerToggle = new ActionBarDrawerToggle(
        		this, drawerLayout, R.drawable.ic_drawer,
                R.string.MainActivity_menu_open, R.string.MainActivity_menu_close) {
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getActionBar().setTitle(title);
            }
            
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getActionBar().setTitle(drawerTitle);
            }
        };

        // Set the drawer toggle as the DrawerListener
        drawerLayout.setDrawerListener(drawerToggle);
        
        getActionBar().setDisplayHomeAsUpEnabled(true);
        getActionBar().setHomeButtonEnabled(true);

		imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		
		// makes sure the volume can be controlled by the student
		setVolumeControlStream(AudioManager.STREAM_MUSIC);
	}

	public void setDrawerIndicatorEnabled(boolean status) {
		drawerToggle.setDrawerIndicatorEnabled(status);
	}
	
	@Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        // Sync the toggle state after onRestoreInstanceState has occurred.
        drawerToggle.syncState();
    }
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }
	
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // pass the event to ActionBarDrawerToggle, if it returns
        // true, then it has handled the app icon touch event
        if (drawerToggle.onOptionsItemSelected(item)) {
			// hide virtual keyboard when opening the drawer
			imm.hideSoftInputFromWindow(drawerLayout.getWindowToken(), 0);
        	return true;
        }

        // in cases where the back arrow in the action bar is visible
        // and the user presses it, we then make sure to execute the
        // exact same code as if the user would have pressed the
        // physical back button
        if (item.getItemId() == android.R.id.home) {
        	onBackPressed();
        }

        return super.onOptionsItemSelected(item);
    }
	
    /* Called whenever we call invalidateOptionsMenu() */
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        // If the nav drawer is open, hide action items related to the content view
        //boolean drawerOpen = drawerLayout.isDrawerOpen(drawerLinear);
        //menu.findItem(R.id.action_websearch).setVisible(!drawerOpen);
        return super.onPrepareOptionsMenu(menu);
    }
	
	private class DrawerItemClickListener implements ListView.OnItemClickListener {
	    @Override
	    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	        selectItem(MenuType.getType(position));
	    }
	}
	
	public enum MenuType {
		START(0),
		EFFORT(1),
		COURSES(2),
		SCHEDULE(3),
		INVENTORY(4),
		SHOP(5),
		ACHIEVEMENTS(6),
		SETTINGS(7);
		
		private int position;
		
		private MenuType(int position) {
			this.position = position;
		}
		
		public int getPosition() {
			return position;
		}
		
		public static MenuType getType(int position) {
			for (MenuType type : MenuType.values()) {
				if (position == type.getPosition()) {
					return type;
				}
			}
			
			return COURSES; // defaults to the effort fragment
		}
	}
	
	private Fragment createFragment(MenuType type) {
		Fragment fragment = null;
		
		switch (type) {
			case START:
				fragment = new StartFragment();
				break;
			case EFFORT:
				fragment = new EffortFragment();
				break;
			case COURSES:
				fragment = new CoursesFragment();
				break;
			case SCHEDULE:
				fragment = new ScheduleFragment();
				break;
			case INVENTORY:
				fragment = new InventoryFragment();
				break;
			case SHOP:
				fragment = new ShopFragment();
				break;
			case ACHIEVEMENTS:
				fragment = new AchievementsFragment();	
				break;
			case SETTINGS:
				fragment = new SettingsFragment();
				break;
		}
		
		return fragment;
	}

	/**
	 *  Swaps fragments in the main content view 
	 */
	private void selectItem(MenuType type) {
        Fragment fragment = createFragment(type);

	    // insert the fragment by replacing any existing fragment
	    fragmentManager.beginTransaction()
				.replace(R.id.content_frame, fragment)
				.addToBackStack(String.valueOf(type.getPosition()))
			    .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
			    .commit();
	    
	    highlightMenuItem(type.getPosition());
	}
	
	/**
	 * Highlights the selected menu item, updates the title
	 * in the action bar, and closes the drawer.
	 * 
	 * @param position
	 */
	public void highlightMenuItem(int position) {
	    drawerList.setItemChecked(position, true);
	    setTitle(menuTitles[position]);
	    drawerLayout.closeDrawer(drawerLinear);
	}

	@Override
	public void setTitle(CharSequence title) {
	    this.title = title;
	    getActionBar().setTitle(title);
	}
	
	@Override
	public void onBackPressed() {
		int stackIndex = fragmentManager.getBackStackEntryCount();
		
		if (stackIndex == 1) {
			showExitDialog();
		} else {
			super.onBackPressed();
			
			// the name indicates the position set by addToBackStack()
			// to identify the given fragment without using a TAG or ID.
			//
			// meaning the right menu item according to what is on the
			// back stack will be highlighted when the student pushes
			// the back button.
			//
			// we need to check for NumberFormatExceptions here in case
			// we are coming a notification, and then the name would be
			// a null.
			try {
				int position = Integer.parseInt(fragmentManager.getBackStackEntryAt(stackIndex - 2).getName());
				highlightMenuItem(position);
			} catch (NumberFormatException e) {}
		}
	}
	
	private void showExitDialog() {		
		(new AlertDialog.Builder(activity))
				.setTitle(getString(R.string.MainActivity_dialog_exit_app_title))		
				.setMessage(getString(R.string.MainActivity_dialog_exit_app_message, getString(android.R.string.ok)))
				.setCancelable(true)
				.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int id) {
						finish();
					}
				})
				.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int id) {
					}
				})
				.setOnDismissListener(new OnDismissListener() {
					@Override
					public void onDismiss(DialogInterface dialog) {
					}
				})
				.create().show();
    }

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);        
		return true;
	}

	// TODO: move this class to a loading screen activity?
    private class ParseTask extends AsyncTask<String, String, String> {

    	private static final String REGISTER = "http://hv-6141.idi.ntnu.no/schooltracker/app_register_student.php";
    	
    	private AlertDialog dialog;
    	private DbHelper db;
    	private boolean studentAdded;

		@Override
		protected void onPreExecute() {
			AlertDialog.Builder builder = new AlertDialog.Builder(activity);
			
			dialog = builder.setTitle(R.string.MainActivity_dialog_loading_app_title)
					.setMessage(R.string.MainActivity_dialog_loading_app_message)
				    .setCancelable(false)
				    .create();
			
			dialog.show();
		}

		@Override
		protected String doInBackground(String... input) {
	        db = new DbHelper(activity);
	        
	        // TODO: remove before release or make sure I'm false!
	        db.recreateTables(false);
	        
	        // TODO: use SharedPreferences/db to ensure that the tables are cached locally
	        initializeCourses();
	        initializeAchievements();
	        initializeStudent();
	        initializeShopItems();
	        initializeRooms();

			return null;
		}

		private void initializeCourses() {
        	ConnectivityManager conManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
    	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
    	    
    	    if (networkInfo != null && networkInfo.isConnected()) {
    	    	Log.e(TAG, "courses: caching..");
    	    	List<Course> courses = CourseParser.parseCourses();
    	    	
    	    	Log.e(TAG, "adding " + courses.size() + " courses to the database..");
    	    	db.cacheCourses(courses);
    	    } else {
    	    	Log.e(TAG, "no internet connection");
    	    }
		}

		private void initializeAchievements() {
	        // check if the achievements have been added to the database
	        List<Achievement> achievements = db.getAchievements(AchievementType.values());
	        if (achievements.size() == 0) {
	        	Log.e(TAG, "achievements: caching..");
	        	achievements.add(new Achievement(
	        			AchievementType.CHEST_1.getId(),
	        			AchievementHandler.CHEST,
	        			"Kiste (1)", "Du fikk din frste kiste!",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.CHEST_10.getId(),
	        			AchievementHandler.CHEST,
	        			"Kiste (10)", "10 kister? Ikke drlig!",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.CHEST_25.getId(),
	        			AchievementHandler.CHEST,
	        			"Kiste (25)", "25! Det hjelper p n.",
	        			10));
	        	achievements.add(new Achievement(
	        			AchievementType.CHEST_50.getId(),
	        			AchievementHandler.CHEST,
	        			"Kiste (50)", "Kister, kister overalt!",
	        			20));
	        	achievements.add(new Achievement(
	        			AchievementType.ATTENDANCE_1.getId(),
	        			AchievementHandler.ATTENDANCE,
	        			"Oppmte (1)", "Du mtte opp til din frste time i r!",
	        			5));
	        	achievements.add(new Achievement(        			
	        			AchievementType.ATTENDANCE_10.getId(),
	        			AchievementHandler.ATTENDANCE,
	        			"Oppmte (10)", "10 nede, prv deg p 15 til.",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.ATTENDANCE_25.getId(),
	        			AchievementHandler.ATTENDANCE,
	        			"Oppmte (25)", "Godt jobba!",
	        			10));
	        	achievements.add(new Achievement(
	        			AchievementType.ATTENDANCE_50.getId(),
	        			AchievementHandler.ATTENDANCE,
	        			"Oppmte (50)", "Du er s og si fast inventar p skolen, ikke sant?",
	        			20));
	        	achievements.add(new Achievement(
	        			AchievementType.COIN_100.getId(),
	        			AchievementHandler.COIN,
	        			"Mynt (100)", "Du har samlet dine frste 100 mynter.",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.COIN_250.getId(),
	        			AchievementHandler.COIN,
	        			"Mynt (250)", "Synd de ikke tar mynter p Hangaren .. enda!",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.COIN_500.getId(),
	        			AchievementHandler.COIN,
	        			"Mynt (500)", "n mynt, to mynter .. FEM HUNDRE mynter.",
	        			10));
	        	achievements.add(new Achievement(
	        			AchievementType.COIN_1000.getId(),
	        			AchievementHandler.COIN,
	        			"Mynt (1000)", "Onkel Skrue, er det deg?",
	        			20));
	        	achievements.add(new Achievement(
	        			AchievementType.TAP_10.getId(),
	        			AchievementHandler.TAP,
	        			"Tapp (10)", "Tapp, tapp, tapp, tapp, tapp, tapp, tapp, tapp, tapp, .. og tapp.",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.TAP_50.getId(),
	        			AchievementHandler.TAP,
	        			"Tapp (50)", "Ett tapp her og ett tapp der, det blir mange tapp det.",
	        			5));
	        	achievements.add(new Achievement(
	        			AchievementType.TAP_100.getId(),
	        			AchievementHandler.TAP,
	        			"Tapp (100)", "Hipp, hipp, hurra for ditt 100. tapp!",
	        			10));
	        	achievements.add(new Achievement(
	        			AchievementType.TAP_150.getId(),
	        			AchievementHandler.TAP,
	        			"Tapp (150)", "N har du vrt .. tapper (JA, DU LESTE RIKTIG!)",
	        			20));
	        	
	        	Log.e(TAG, "adding " + achievements.size() + " achievements to the database..");
	        	db.addAchievements(achievements);
	        }
		}

		private void initializeStudent() {
	        Student student = db.getStudent(1);
	        
			// we assume it's true so the error message
			// isn't displayed otherwise
			studentAdded = true;
	        
	        // if there isn't a student present in the system,
	        // we generate one with the uuid from the webserver
	        if (student == null) {
	        	studentAdded = false;
	        	
	        	ConnectivityManager conManager = (ConnectivityManager) activity.getSystemService(Context.CONNECTIVITY_SERVICE);
        	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();

        	    // we check if the device has some form of connection
        	    // (assumes internet connection)
        	    if (networkInfo != null && networkInfo.isConnected()) {
					String uuid = null;
					InputStream is = null;
					
			    	// http post
					try {
			    		HttpClient httpclient = new DefaultHttpClient();
						HttpPost httppost = new HttpPost(REGISTER);
						HttpResponse response = httpclient.execute(httppost);
						HttpEntity entity = response.getEntity();
						is = entity.getContent();
					} catch (ClientProtocolException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					} catch (IllegalStateException e) {
						e.printStackTrace();
					}
			    	
			    	// convert response to string (uuid)
		    		BufferedReader reader;
					try {
						reader = new BufferedReader(new InputStreamReader(is, ConstantHandler.CHARSET), 8);
						uuid = reader.readLine();
					} catch (UnsupportedEncodingException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					} finally {
						try {
							is.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
					
					if (uuid != null) {
						db.addStudent(uuid);
						studentAdded = true;
					}
        	    }
	        }
		}

		private void initializeShopItems() {
	        List<ShopItem> items = db.getShopItems();
	        if (items.size() == 0) {
	        	Log.e(TAG, "items: caching..");
	        	items.add(new ShopItem(
	        			ItemType.ICE_AXE, IceAxe.NAME + " (1)",
	        			"n " + IceAxe.NAME.toLowerCase(ConstantHandler.NORWEGIAN) + " som kan brukes n gang.",
	        			50, 1, 0));
	        	items.add(new ShopItem(
	        			ItemType.ICE_AXE, IceAxe.NAME + " (5)",
	        			"5 " + IceAxe.NAME.toLowerCase(ConstantHandler.NORWEGIAN) + "er som kan brukes n gang hver.",
	        			200, 5, 0));
	        	items.add(new ShopItem(
	        			ItemType.ICE_AXE, IceAxe.NAME + " (10)",
	        			"10 " + IceAxe.NAME.toLowerCase(ConstantHandler.NORWEGIAN) + "er som kan brukes n gang hver.",
	        			380, 10, 0));
	        	
	        	Log.e(TAG, "adding " + items.size() + " shop items to the database..");
	        	db.addShopItems(items);
	        }
		}
		
		private void initializeRooms() {
        	ConnectivityManager conManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
    	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
    	    
    	    if (networkInfo != null && networkInfo.isConnected()) {
    	    	Log.e(TAG, "rooms: caching..");
    	    	List<Room> rooms = RoomParser.parseRooms();
    	    	
    	    	Log.e(TAG, "adding " + rooms.size() + " rooms to the database..");
    	    	db.cacheRooms(rooms);
    	    }
		}

		@Override
		protected void onProgressUpdate(String... values) {
		}

		@Override
		protected void onPostExecute(String results) {
			dialog.cancel();
			
    	    if (!studentAdded) {
				AlertDialog.Builder builder = new AlertDialog.Builder(activity);
				builder.setTitle(R.string.MainActivity_dialog_no_internet_student_creation_title)
						.setMessage(R.string.MainActivity_dialog_no_internet_student_creation_message)
					    .setCancelable(false)
					    .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
					    		public void onClick(DialogInterface dialog, int id) {
				    				finish();
					    		}
				       }
			    );
				builder.create().show();
			} else {
				// this piece of code isn't really necessary except in
				// cases where activities are in progress and we
				// update/patch the app. this will cause PositionService
				// to close and no positions to be captured. therefore
				// we make sure that PositionService is running even though
				// the service is supposed to start on boot and automatically
				// be restarted by the system if it's killed.
				SchedulingService.scheduleAlarms(activity);
				
				if (getIntent().getExtras() != null) {
					selectFragmentFromNotification();
				} else {
					// loads the default fragment into the activity
					selectItem(MenuType.START);
				}
			}    	    
		}

    }
    
    @Override
    protected void onNewIntent (Intent intent) {
    	setIntent(intent);
    }
    
    @Override
    public void onResume() {
    	super.onResume();
    	
    	// this causes the correct fragment to be loaded by this activity
    	// if the activity is either brought forward or started by a notification
    	selectFragmentFromNotification();
    }
    
    private void selectFragmentFromNotification() {
	    // if the app was started/brought forward by a notification
	    // we decide what fragment we want to select
		Bundle bundle = getIntent().getExtras();			
		if (bundle != null) {
			for (String type : NotificationHandler.NOTIFICATION_TYPE) {
				int fragment = bundle.getInt(type);
				
				if (fragment == NotificationHandler.ACHIEVEMENT) {
					bundle.remove(type);
					selectItem(MenuType.ACHIEVEMENTS);
					break;
				} else if (fragment == NotificationHandler.LOOT) {
					bundle.remove(type);
					selectItem(MenuType.INVENTORY);
					break;
				}
			}
		}
    }
}