package no.skarasoftwaredevelopment.schooltracker.gui;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.MyCourseAdapter;
import no.skarasoftwaredevelopment.schooltracker.handlers.ToastHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Course;
import no.skarasoftwaredevelopment.schooltracker.services.PositionService;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;
import android.widget.TextView;

public class MyCoursesFragment extends Fragment {

	public static final String TAG = "MyCoursesFragment";
	
	private Activity activity;
	private MyCourseAdapter adapter;
	private DbHelper db;
	private Course course;
    private PositionService serviceBinder;
    private ServiceConnection serviceConncetion;
	
	private TextView empty;
	private ListView list;
	
    public MyCoursesFragment() {
        // Empty constructor required for fragment subclasses
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_my_courses, container, false);        
		
        activity = getActivity();
        db = new DbHelper(getActivity());
        serviceConncetion = new LocalServiceConnection();
        
        empty = (TextView) rootView.findViewById(R.id.fragment_my_courses_emptyTextView);
        
		list = (ListView) rootView.findViewById(R.id.fragment_my_courses_listListView);
		list.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, final View view, final int position, long id) {
				final Drawable oldBackground = view.getBackground();
				view.setBackground(activity.getResources().getDrawable(android.R.color.holo_blue_light));

				AlertDialog.Builder builder = new AlertDialog.Builder(activity);

				builder.setTitle(R.string.MyCoursesFragment_dialog_remove_title)
						.setMessage(R.string.MyCoursesFragment_dialog_remove_message)
						.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog, int id) {
								course = (Course) list.getItemAtPosition(position);
								new DeleteTask((Course) list.getItemAtPosition(position)).execute();
							}
						})
						.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog,
									int id) {
								// closes the dialog
							}
						})
						.setOnDismissListener(new OnDismissListener() {
							@Override
							public void onDismiss(DialogInterface dialog) {
								view.setBackground(oldBackground);
							}
						})
						.create().show();
			}
		});
		
		// TODO: should we do the database connections in an async task?
		adapter = new MyCourseAdapter(getActivity(), db.getStudentCourses(1)); // TODO: dynamically find studentId
		list.setAdapter(adapter);
        
		// show explanation if the list is empty (no items in the inventory)
		// we need to do it this way instead of using the built in functions
		// in the ListFragment due to the menu drawer (overrides the listener)
		if (adapter.getCount() == 0) {
			empty.setVisibility(TextView.VISIBLE);
		}
		
        return rootView;
    }
    
    /**
     * This class is used to communicate with {@code PositionService}.
     * When the client is connected to the service, it provides the
     * service with the course, and the service removes all tasks
     * related to the course the student wants to stop tracking.
     * Afterwards the class cleans up the connection to the service.
     * 
     * @author Tommy
     *
     */
    private class LocalServiceConnection implements ServiceConnection {

		@Override
		public void onServiceConnected(ComponentName name, IBinder service) {
        	serviceBinder = ((PositionService.LocalBinder) service).getService();
        	
        	if (course != null) {
        		serviceBinder.cancelAllTasks(course.getCode());
        	}
        	
        	activity.unbindService(serviceConncetion);
		}

		@Override
		public void onServiceDisconnected(ComponentName name) {
			serviceBinder = null;
    		course = null;
		}

    }
    
    /**
     * If {@code PositionService} is running and we have not previously
     * bound to this service, we try to bind to it.
     */
    private void bindLocationService() {
    	if (PositionService.isRunning && serviceBinder == null) {
    		Log.e(TAG, "trying to bind service..");
    		
    		Intent intent = new Intent(activity, PositionService.class);    		
	        activity.bindService(intent, serviceConncetion, 0);
    	}
    }
    
    private class DeleteTask extends AsyncTask<String, String, String> {

    	private Course course;
        private AlertDialog dialog;
        
        public DeleteTask(Course course) {
        	this.course = course;
        }
    	
        @SuppressLint("InflateParams")
		@Override
		protected void onPreExecute() {
			View v = activity.getLayoutInflater().inflate(R.layout.dialog_loader, null);
			dialog = new AlertDialog.Builder(activity)
					.setView(v)
					.setCancelable(false)
					.setTitle(R.string.MyCoursesFragment_dialog_removing_title).create();
			dialog.show();
		}
		
		@Override
		protected String doInBackground(String... input) {
			bindLocationService();
			db.removeCourse(1, course.getCode()); // TODO: get studentId dynamically
			return null;
		}
		
		@Override
		protected void onProgressUpdate(String... values) {
		}
	
		@Override
		protected void onPostExecute(String results) {
			adapter.remove(course);
			adapter.notifyDataSetChanged();
			
			if (adapter.getCount() == 0) {
				empty.setVisibility(TextView.VISIBLE);
			}
			
			dialog.cancel();
			ToastHandler.shortMsg(activity, R.string.MyCoursesFragment_toast_removed);
		}

    }

}