package no.skarasoftwaredevelopment.schooltracker.gui;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.FormatHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Weekday;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

public class ScheduleFragment extends Fragment {

	public static final String TAG = "ScheduleFragment";
	public static final int NUMBER_OF_WEEKDAYS = 5;

	private WeekdayPagerAdapter adapter;
	private ViewPager viewPager;
	private int numberOfCourses;

	public ScheduleFragment() {
		// Empty constructor required for fragment subclasses
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View rootView = inflater.inflate(R.layout.fragment_schedule, container, false);
		
		viewPager = (ViewPager) rootView.findViewById(R.id.pager);
		new LoadWeekdaysTask().execute();

		return rootView;
	}
	
    private class LoadWeekdaysTask extends AsyncTask<String, String, String> {
    	
    	private Activity activity;
    	private AlertDialog dialog;
    	private Weekday[] weekdays;
    	
    	@SuppressLint("InflateParams")
		@Override
		protected void onPreExecute() {
			activity = getActivity();
			
			// show loader dialog
			View v = activity.getLayoutInflater().inflate(R.layout.dialog_loader, null);
			dialog = new AlertDialog.Builder(activity)
					.setView(v)
					.setCancelable(false)
					.setTitle(R.string.ScheduleFragment_dialog_getting_schedule_title).create();
			dialog.show();
			
			weekdays = new Weekday[NUMBER_OF_WEEKDAYS];
		}
		
		@Override
		protected String doInBackground(String... input) {			
			// today's date
	    	Calendar now = Calendar.getInstance(ConstantHandler.NORWEGIAN);
	    	int weekNumber = now.get(Calendar.WEEK_OF_YEAR);
	    	
	    	DbHelper db = new DbHelper(activity);
	    	List<String> courses = db.getStudentCourseCodes(1); // TODO: get studentId dynamically
	    	numberOfCourses = courses.size();

	    	for (int i = now.getFirstDayOfWeek() - 2; i < NUMBER_OF_WEEKDAYS; i++) { // firstDayOfWeek = 2 (Java), NTNU = 0
	    		// we get all the attendances for the given day in a given week
	    		ArrayList<Attendance> attendances = (ArrayList<Attendance>) db.getStudentAttendances(1, i, weekNumber); // TODO: get studentId dynamically   		
	    		
	    		now.set(Calendar.DAY_OF_WEEK, i + 2); // add 2 to make it compatible with NTNU's formatting
	    		weekdays[i] = new Weekday(now.getTime(), attendances); // TODO: get studentId dynamically
	    	}
			
			return null;
		}
		
		@Override
		protected void onProgressUpdate(String... values) {
		}
	
		@Override
		protected void onPostExecute(String results) {
			adapter = new WeekdayPagerAdapter(getChildFragmentManager(), weekdays);
			viewPager.setAdapter(adapter);

			// today's date
	    	Calendar now = Calendar.getInstance();
	    	int dayNumber = now.get(Calendar.DAY_OF_WEEK);
	    	
	    	// we subtract 2 here because monday = 2 in Java
	    	// and the first index of the viewpager is 0
	    	dayNumber -= 2;
	    	viewPager.setCurrentItem(dayNumber);
	    	
	    	dialog.cancel();
		}

    }
    
	public class WeekdayPagerAdapter extends FragmentStatePagerAdapter  {

		private Weekday[] weekdays;
		
		public WeekdayPagerAdapter(FragmentManager fm, Weekday[] weekdays) {
			super(fm);
			this.weekdays = weekdays;			
		}

		@Override
		public Fragment getItem(int i) {
			Fragment fragment = new WeekdayFragment();
			Bundle args = new Bundle();
			
			ArrayList<Attendance> attendances = weekdays[i].getAttendances();
			args.putSerializable(WeekdayFragment.ARG_ATTENDANCES, attendances);
			args.putSerializable(WeekdayFragment.ARG_NUMBER_OF_COURSES, numberOfCourses);
			fragment.setArguments(args);

			return fragment;
		}

		@Override
		public int getCount() {
			return NUMBER_OF_WEEKDAYS;
		}

		@Override
		public CharSequence getPageTitle(int position) {			
			return FormatHandler.weekdayFormat.format(weekdays[position].getDate());
		}

	}
	
	@Override
	public void onResume() {
		super.onResume();
	}

}