package no.skarasoftwaredevelopment.schooltracker.gui;

import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.StudyProgramAdapter;
import no.skarasoftwaredevelopment.schooltracker.handlers.ToastHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.UploadDbHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudyProgram;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.res.Resources;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TextView;

public class SettingsFragment extends Fragment {
	
	public static final String TAG = "SettingsFragment";
	
	private Activity activity;
	private StudyProgram oldProgram;
	private DbHelper db;
	private Student student;
	
    public SettingsFragment() {
        // Empty constructor required for fragment subclasses
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_settings, container, false);
        
        activity = getActivity();
        
        final Spinner pick = (Spinner) rootView.findViewById(R.id.fragment_settings_pickStudyProgramSpinner);
        final Button upload = (Button) rootView.findViewById(R.id.fragment_settings_uploadButton);
        TextView description = (TextView) rootView.findViewById(R.id.fragment_settings_studyProgramDescription);
        TextView identification = (TextView) rootView.findViewById(R.id.fragment_settings_identificationTextView);
        
        // the student can't change the study program after adding courses
        // because this could cause a problem in cases where course activities
        // only applies to the previous study program and not the newly selected one
        db = new DbHelper(activity);
        List<String> courses = db.getStudentCourseCodes(1); // TODO: get studentId dynamically
        
        if (!courses.isEmpty()) {
        	pick.setEnabled(false);
        	description.setText(R.string.SettingsFragment_text_description_disabled);
        } else {
        	description.setText(R.string.SettingsFragment_text_description_enabled);
        	
            pick.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
    			@Override
    			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
    				StudyProgram newProgram = (StudyProgram) pick.getSelectedItem();
    				
    				// no point in saving the study program again if it's the same as before
    				// this is also handled by the android framework, but not for the first
    				// time the fragment is loaded
    				if (position > 0 && (oldProgram == null || !oldProgram.equals(newProgram))) {
    					// considering the how small the interaction with the database is
    					// we just run it on the GUI thread instead of a background thread
    					DbHelper db = new DbHelper(activity);
    					db.updateStudyProgram(1, newProgram.getCode()); // TODO: get studentId dynamically
    				}
    			}

    			@Override
    			public void onNothingSelected(AdapterView<?> parent) {
    			}
            });
        }

        // setting up the list of study programs
        Resources res = activity.getResources();
        
        String[] programAcronyms = res.getStringArray(R.array.SettingsFragment_spinner_pick_program_acronyms);
        String[] programDescriptions = res.getStringArray(R.array.SettingsFragment_spinner_pick_program_descriptions);
        
        List<StudyProgram> programs = new ArrayList<StudyProgram>();
        for (int i = 0; i < programAcronyms.length; i++) {
        	programs.add(new StudyProgram(programAcronyms[i], programDescriptions[i]));
        }
        
        StudyProgramAdapter adapter = new StudyProgramAdapter(activity, programs);
        pick.setAdapter(adapter);

        // if the user have chosen a study program we want
        // to select the correct one to display 
        student = db.getStudent(1); // TODO: get studentId dynamically
        StudyProgram program = new StudyProgram(student.getStudyProgram(), ""); // only the program code is checked in the equals-method
    	int index = programs.indexOf(program);
    	
    	if (index > 0) {
    		pick.setSelection(index);
    		// prevents us from re-saving the same study program
    		// when the fragment is loaded
    		oldProgram = programs.get(index);
    	}

        upload.setOnClickListener(new View.OnClickListener() {			
			@Override
			public void onClick(View v) {
				upload.setEnabled(false);
				
				ConnectivityManager conManager = (ConnectivityManager) activity.getSystemService(Context.CONNECTIVITY_SERVICE);
        	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
        	    
        	    if (networkInfo != null && networkInfo.isConnected()) {
        	    	new UploadTask().execute();
        	    } else {
        	    	ToastHandler.shortMsg(activity, R.string.AddCoursesFragment_toast_no_internet_connection);
        	    }
        	    
        	    upload.setEnabled(true);
			}
		});
        
        // displays the unique student identification for use
        // in online questionnaires etc. so we can correlate
        // the database uploaded with the answers given by
        // the student
        identification.setText(student.getUUID());
        
        return rootView;
    }

    private class UploadTask extends AsyncTask<String, String, String> implements Runnable {
    	
    	private AlertDialog dialog;
    	
		@SuppressLint("InflateParams")
		@Override
		protected void onPreExecute() {
			View v = activity.getLayoutInflater().inflate(R.layout.dialog_loader, null);
			dialog = new AlertDialog.Builder(activity)
					.setView(v)
					.setCancelable(false)
					.setTitle(R.string.SettingsFragment_dialog_uploading).create();
			dialog.show();
        }
		
		@Override
		protected String doInBackground(String... input) {
			UploadDbHandler.takeBackup(db.getPath(), student.getUUID());
			return null;
		}
		
		@Override
		protected void onProgressUpdate(String... values) {
		}
	
		@Override
		protected void onPostExecute(String results) {			
			dialog.cancel();
			ToastHandler.longMsg(activity, R.string.SettingsFragment_toast_upload_complete);
		}

		@Override
		public void run() {
		}

    }
    
}