package no.skarasoftwaredevelopment.schooltracker.gui;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.database.DbHelper.DbStatus;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.ShopAdapter;
import no.skarasoftwaredevelopment.schooltracker.handlers.SoundHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ToastHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.ShopItem;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class ShopFragment extends Fragment {
	
	public static final String TAG = "ShopFragment";
	
	private int coins;
	private Activity activity;
	
	private TextView amount;
	private ListView list;
	
    public ShopFragment() {
        // Empty constructor required for fragment subclasses
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
    	View rootView = inflater.inflate(R.layout.fragment_shop, container, false);
    	
    	activity = getActivity();
        
        list = (ListView) rootView.findViewById(R.id.fragment_shop_listListView);
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				ShopItem item = (ShopItem) list.getItemAtPosition(position);

				if (item.getPrice() <= coins) {
					showConfirmationDialog(item);
				} else {
					ToastHandler.shortMsg(activity, R.string.ShopFragment_toast_insufficient_coins);
				}
			}
		});
        
		// TODO: should we do the database connections in an async task?
		DbHelper db = new DbHelper(getActivity());
		
        Student student = db.getStudent(1); // TODO: get studentId dynamically
		coins = student.getCoins();
		
		list.setAdapter(new ShopAdapter(getActivity(), db.getShopItems(), coins));
		
		// makes android call onCreateOptionsMenu for this fragment
		// which in turn displays the amount of coins the student currently has
		setHasOptionsMenu(true);
		
		// show explanation if the list is empty (nothing to buy in the shop)
		// we need to do it this way instead of using the built in functions
		// in the ListFragment due to the menu drawer (overrides the listener)
		if (list.getAdapter().getCount() == 0) {
			TextView empty = (TextView) rootView.findViewById(R.id.fragment_shop_emptyTextView);
			empty.setVisibility(TextView.VISIBLE);
		}
		
        return rootView;
    }
    
    private void showConfirmationDialog(final ShopItem item) {
		(new AlertDialog.Builder(activity))
				.setTitle(getString(R.string.ShopFragment_dialog_transaction_title, item.getName()))		
				.setMessage(getString(R.string.ShopFragment_dialog_transaction_message, item.getPrice()))
				.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int id) {
						DbHelper db = new DbHelper(activity);
						DbStatus status = db.addItem(1, item); // TODO: get studentId dynamically
						
						if (status == DbStatus.OK) {
							// deduct item price from the coins
							db.updateCoins(1, -item.getPrice());
							
							// refresh the total in the upper right corner
					        Student student = db.getStudent(1); // TODO: get studentId dynamically
							coins = student.getCoins();
							amount.setText(String.valueOf(coins));
							
							SoundHandler.playSound(activity, R.raw.win);
							ToastHandler.shortMsg(activity, getString(R.string.ShopFragment_toast_transaction_completed, item.getName()));
						}
					}
				})
				.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog,
							int id) {
						// do nothing :D
					}
				})
				.setOnDismissListener(new OnDismissListener() {
					@Override
					public void onDismiss(DialogInterface dialog) {
						// update the coin amount for the adapter and redraw the items
						// in case the student used up enough coins to make other items
						// unavailable
						ShopAdapter adapter = (ShopAdapter) list.getAdapter();
						adapter.setCoins(coins);
					}
				})
				.create().show();
    }
    
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
    	inflater.inflate(R.menu.main, menu);
       
		amount = new TextView(getActivity());
		amount.setText(String.valueOf(coins));
		amount.setTextColor(getResources().getColor(android.R.color.white));
		amount.setTextSize(18);
		menu.add(1, View.NO_ID, 1, "").setActionView(amount).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);

		ImageView iv = new ImageView(getActivity());
		iv.setImageResource(R.drawable.coins);
		menu.add(1, View.NO_ID, 2, "").setActionView(iv).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
    }

}