package no.skarasoftwaredevelopment.schooltracker.gui;

import java.util.ArrayList;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.gui.adapters.WeekdayAdapter;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.services.PositionService;
import no.skarasoftwaredevelopment.schooltracker.services.PositionTaskService;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.TextView;

public class WeekdayFragment extends Fragment {
	
	public static final String TAG = "WeekdayFragment";
	public static final String ARG_ATTENDANCES = "no.skarasoftwaredevelopment.schooltracker.gui.OBJECT";
	public static final String ARG_NUMBER_OF_COURSES = "no.skarasoftwaredevelopment.schooltracker.gui.NUMBER_OF_COURSES";

	private WeekdayAdapter adapter;
	
	private ListView list;
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// the last two arguments ensure LayoutParams are inflated properly
		View rootView = inflater.inflate(R.layout.fragment_weekday, container, false);
		
		Activity activity = getActivity();
		
		Bundle args = getArguments();
		@SuppressWarnings("unchecked")
		ArrayList<Attendance> attendances = (ArrayList<Attendance>) args.getSerializable(ARG_ATTENDANCES);
		int numberOfCourses = args.getInt(ARG_NUMBER_OF_COURSES);

		list = (ListView) rootView.findViewById(R.id.fragment_weekday_attendanceListView);
		
		adapter = new WeekdayAdapter(activity, attendances);
		list.setAdapter(adapter);
		
		// show explanation if the list is empty (no courses added yet)
		// we need to do it this way instead of using the built in functions
		// in the ListFragment due to the menu drawer (overrides the listener)
		if (list.getAdapter().getCount() == 0) {
			TextView empty = (TextView) rootView.findViewById(R.id.fragment_weekday_emptyTextView);
			
			// if the student has added courses, but there is no activities
			// today, we show a different message to the student
			if (numberOfCourses > 0) {
				empty.setText(R.string.ScheduleFragment_text_no_activity);
			}
			
			empty.setVisibility(TextView.VISIBLE);
		}
		
		// we tell the BroadcastManager what Action to look for with the IntentFilter
		// and how it should be treated with the BroadcastReceiver
        IntentFilter filter = new IntentFilter(PositionTaskService.ACTION_BROADCAST);
        PositionReceiver receiver = new PositionReceiver();
        LocalBroadcastManager.getInstance(activity).registerReceiver(receiver, filter);
		
		return rootView;
	}
	
	/**
	 * This class is used to receive status updates
	 * from {@code PositionTaskService}.
	 * 
	 * @author Tommy
	 *
	 */
    private class PositionReceiver extends BroadcastReceiver {

        private PositionReceiver() {
        }

        // this method is automatically called when receiving an Intent broadcast
        public void onReceive(Context context, Intent intent) {     	
        	Attendance attendance = (Attendance) intent.getSerializableExtra(PositionService.EXTRA_ATTENDANCE);        	
        	adapter.updateAttendance(attendance);
        }

    }
    
}