package no.skarasoftwaredevelopment.schooltracker.gui.adapters;

import java.util.Calendar;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.handlers.FormatHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;

import org.joda.time.DateTime;
import org.joda.time.Days;
import org.joda.time.Hours;
import org.joda.time.Minutes;

import android.app.Activity;
import android.content.res.Resources;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

public class WeekdayAdapter extends ArrayAdapter<Attendance> {

	public static final String TAG = "WeekdayAdapter";
	
	private Activity activity;
	
	public WeekdayAdapter(Activity activity, List<Attendance> attendances) {
		super(activity, 0, attendances);
		this.activity = activity;
	}
	
	public void updateAttendance(Attendance attendance) {
		int count = getCount();
		for (int i = 0; i < count; i++) {
			Attendance a = getItem(i);
			if (a.equals(attendance)) {
				a.setInside(attendance.isInside());
				break;
			}
		}		
		notifyDataSetChanged();
	}
	
	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		// if we weren't given a view, inflate one
		if (convertView == null) {
			convertView = activity.getLayoutInflater().inflate(R.layout.list_item_weekday_attendance, parent, false);
		}
		
		// configure the view for this Attendance
		Attendance attendance = getItem(position);
		
		TextView description = (TextView) convertView.findViewById(R.id.list_item_weekday_attendance_descriptionTextView);
		description.setText(attendance.getType().toString());

		TextView course = (TextView) convertView.findViewById(R.id.list_item_weekday_attendance_courseTextView);
		course.setText(attendance.getCourseName() + " (" + attendance.getCourseCode() + ")");
    	
		// display which room(s) the activity takes place in
    	// and format it accordingly to the number of rooms
    	String room = "";
    	List<Room> rooms = attendance.getRooms();
    	int numberOfRooms = rooms.size();
    	Room r = null;
    	
    	Resources res = activity.getResources();
    	
    	// used to join two rooms or to concatenate
    	// the last room onto the rest (Oxford comma)
    	String joint = res.getString(R.string.WeekdayAdapter_text_sentence_joint);
    	
    	if (numberOfRooms == 1) {
    		room = rooms.get(0).getName();
    	} else if (numberOfRooms == 2) {
    		room = rooms.get(0).getName() + " " + joint + " " + rooms.get(1).getName();
    	} else {
	    	for (int i = 0; i < numberOfRooms; i++) {
	    		r = rooms.get(i);
	    		if (i + 1 < numberOfRooms) {
	    			room += r.getName() + ", ";
	    		} else {
	    			room += joint + " " + r.getName();
	    		}
	    	}
    	}

    	// figure out how long it is until the activity starts
    	// or since it started before it has passed
    	Calendar nowCal = Calendar.getInstance();

    	DateTime now = new DateTime(nowCal.getTime());
    	DateTime start = new DateTime(attendance.getStartDate().getTime());
    	
    	int dayDiff = Days.daysBetween(now, start).getDays();
    	int hourDiff = Hours.hoursBetween(now, start).getHours();
    	int minuteDiff = Minutes.minutesBetween(now, start).getMinutes();
    	
    	String time = "";
    	String message = "";
    	
    	if (dayDiff >= 1) {
    		time = res.getQuantityString(R.plurals.WeekdayAdapter_text_days, dayDiff, dayDiff);
    		message = res.getString(R.string.WeekdayAdapter_text_future, time, room);
    	} else if (hourDiff >= 1) {
    		time = res.getQuantityString(R.plurals.WeekdayAdapter_text_hours, hourDiff, hourDiff);
    		message = res.getString(R.string.WeekdayAdapter_text_future, time, room);
    	} else if (hourDiff == 0) {
    		if (minuteDiff >= 1) {
    			time = res.getQuantityString(R.plurals.WeekdayAdapter_text_minutes, minuteDiff, minuteDiff);
    			message = res.getString(R.string.WeekdayAdapter_text_future, time, room);
    		} else if (minuteDiff == 0) {
    			message = res.getString(R.string.WeekdayAdapter_text_now, room);
    		} else if (minuteDiff <= -1) {
				time = res.getQuantityString(R.plurals.WeekdayAdapter_text_minutes, (-1) * minuteDiff, (-1) * minuteDiff);
				message = res.getString(R.string.WeekdayAdapter_text_past, time, room);
    		}
		} else if (hourDiff <= -1) {
			time = res.getQuantityString(R.plurals.WeekdayAdapter_text_hours, (-1) * hourDiff, (-1) * hourDiff);
			message = res.getString(R.string.WeekdayAdapter_text_past, time, room);
		}
    	
    	TextView timer = (TextView) convertView.findViewById(R.id.list_item_weekday_attendance_timerTextView);
    	timer.setText(message);
		
		TextView hour = (TextView) convertView.findViewById(R.id.list_item_weekday_attendance_hourTextView);
		hour.setText(FormatHandler.hourFormat.format(attendance.getStartDate().getTime()) + " - " +
					FormatHandler.hourFormat.format(attendance.getEndDate().getTime()));
	
		LinearLayout container = (LinearLayout) convertView.findViewById(R.id.list_item_weekday_attendance_containerLinearLayout);
		
		// various states an activity can be in, as shown in the ScheduleFragment/WeekdayFragment
		if (attendance.isInside() && isInProgress(attendance, nowCal)) {
			// green background
			container.setBackgroundResource(R.drawable.background_attending);
			timer.setText(res.getString(R.string.WeekdayAdapter_text_attending));
		} else if (attendance.hasPassed() && attendance.hasAttended()) {
			// green background
			container.setBackgroundResource(R.drawable.background_attending);
			timer.setText(res.getString(R.string.WeekdayAdapter_text_did_attend));
		} else if (!attendance.hasPassed() && !attendance.isInside() && isInProgress(attendance, nowCal)) {
			// yellow background
			container.setBackgroundResource(R.drawable.background_active_not_attending);
		} else if ((attendance.hasPassed() || nowCal.after(attendance.getEndDate())) && !attendance.hasAttended()) {
			// red background
			container.setBackgroundResource(R.drawable.background_not_attending);
			timer.setText(res.getString(R.string.WeekdayAdapter_text_did_not_attend));
		} else {
			// white background
			container.setBackgroundResource(R.drawable.background_normal); 
		}
				
		return convertView;
	}
	
	/**
	 * Helper method used to decide if the activity
	 * has started and is in progress.
	 * 
	 * @param attendance
	 * @param now
	 * @return
	 */
	private boolean isInProgress(Attendance attendance, Calendar now) {
		return now.after(attendance.getStartDate()) && now.before(attendance.getEndDate());
	}

}