package no.skarasoftwaredevelopment.schooltracker.handlers;

import no.skarasoftwaredevelopment.schooltracker.R;
import no.skarasoftwaredevelopment.schooltracker.gui.MainActivity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.support.v4.app.NotificationCompat;

public class NotificationHandler {
	
	public static final String EXTRA_NOTIFY_ACHIEVEMENT = "no.skarasoftwaredevelopment.schooltracker.handlers.NOTIFY_ACHIEVEMENT";
	public static final String EXTRA_NOTIFY_LOOT = "no.skarasoftwaredevelopment.schooltracker.handlers.NOTIFY_ACHIEVEMENT";
	public static final int ACHIEVEMENT = 1;
	public static final int LOOT = 2;
	public static final int NETWORK = 3;
	public static final String[] NOTIFICATION_TYPE;
	
	static {
		NOTIFICATION_TYPE = new String[2];
		NOTIFICATION_TYPE[0] = EXTRA_NOTIFY_ACHIEVEMENT;
		NOTIFICATION_TYPE[1] = EXTRA_NOTIFY_LOOT;
	}
	
	/**
	 * Used to show a notification when the student gets (a) new achievement(s).
	 * 
	 * @param context
	 * @param achievements
	 */
	public static void notifyAchievement(Context c, int achievements) {
		String title = c.getResources().getQuantityString(R.plurals.NotificationHandler_notify_achievement_title, achievements, achievements);
		String content = c.getResources().getQuantityString(R.plurals.NotificationHandler_notify_achievement_content, achievements, achievements);
		
		Intent i = createIntent(c, EXTRA_NOTIFY_ACHIEVEMENT, ACHIEVEMENT);
		show(c, i, R.drawable.ic_stat_notify_achievement, title, content, ACHIEVEMENT);
	}
	
	/**
	 * Used to show a notification when the student gets a new chest.
	 * 
	 * @param c
	 */
	public static void notifyLoot(Context c) {
		String title = c.getResources().getString(R.string.NotificationHandler_notify_chest_title);
		String content = c.getResources().getString(R.string.NotificationHandler_notify_chest_content);
		
		Intent i = createIntent(c, EXTRA_NOTIFY_LOOT, LOOT);
		show(c, i, R.drawable.ic_stat_notify_loot, title, content, LOOT);
	}
	
	/**
	 * Used to show a notification when the student haven't enabled gps/wifi/mobile network.
	 * 
	 * @param c
	 */
	public static void notifyNetwork(Context c) {
		String title = c.getResources().getString(R.string.NotificationHandler_notify_network_title);
		String content = c.getResources().getString(R.string.NotificationHandler_notify_network_content);
		
		Intent i = new Intent(); // needed to remove the notification after it was clicked
		show(c, i, R.drawable.ic_stat_notify_global_warning, title, content, NETWORK);
	}
	
	private static Intent createIntent(Context c, String name, int value) {
		Intent i = new Intent(c, MainActivity.class);
		i.setAction(Intent.ACTION_MAIN);
		i.addCategory(Intent.CATEGORY_LAUNCHER);
		i.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
		i.putExtra(name, value);
		
		return i;
	}
	
	private static void show(Context c, Intent i, int iconResId, String title, String content, int type) {
		PendingIntent intent = PendingIntent.getActivity(c, type, i, PendingIntent.FLAG_UPDATE_CURRENT);
		
		Notification notification = new NotificationCompat.Builder(c)
				.setSmallIcon(iconResId)
				.setContentTitle(title)
				.setContentText(content)
				.setStyle(new NotificationCompat.BigTextStyle()
						.bigText(content))
				.setAutoCancel(true) // clear notification after click
				.setDefaults(Notification.DEFAULT_ALL)
				.setContentIntent(intent)
				.build();
		
		NotificationManager notifyManager = (NotificationManager) c.getSystemService(Context.NOTIFICATION_SERVICE);
		notifyManager.notify(type, notification);
	}
	
	/**
	 * Allows us to programmatically remove a given notification type.
	 * 
	 * @param c
	 * @param type
	 */
	public static void removeNotification(Context c, int type) {
		NotificationManager notifyManager = (NotificationManager) c.getSystemService(Context.NOTIFICATION_SERVICE);
		notifyManager.cancel(type);
	}
	
}