package no.skarasoftwaredevelopment.schooltracker.parsers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.pojo.Course;
import android.util.JsonReader;

public final class CourseParser {

	//public static final String COURSES = "http://www.ime.ntnu.no/api/course/-"; // NTNU IME API
	public static final String COURSES = "http://hv-6141.idi.ntnu.no/schooltracker/courses.json";

	public static List<Course> parseCourses() {
		URL url = null;
		BufferedReader in = null;
		JsonReader reader = null;
		List<Course> courses = null;
		
		try {
			url = new URL(COURSES);
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginObject();
			reader.nextName(); // course
			
			courses = readCourseArray(reader);
			
			// let's skip the "cache" and "request" sections in the JSON data
			
			reader.nextName(); // cache
			reader.beginObject();
			while (reader.hasNext()) {
				reader.skipValue();
			}
			reader.endObject();
			
			reader.nextName(); // request
			reader.beginObject();
			while (reader.hasNext()) {
				reader.skipValue();
			}
			reader.endObject();

			reader.endObject();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {}
			}
		}
	
		return courses;
	}

	private static List<Course> readCourseArray(JsonReader reader) throws IOException {
		List<Course> courses = new ArrayList<Course>();

		reader.beginArray();
		while (reader.hasNext()) {
			courses.add(readRoom(reader));
		}
		reader.endArray();
		
		return courses;
	}
	
	private static Course readRoom(JsonReader reader) throws IOException {
		String code = null;
		String name = null;
		String norwegianName = null;
		String newNorwegianName = null;
		String englishName = null;
		int versionCode = -1;

		reader.beginObject();
		while (reader.hasNext()) {
			String nextName = reader.nextName();
			if (nextName.equals("code")) {
				code = reader.nextString();
			} else if (nextName.equals("name")) {
				name = reader.nextString();
			} else if (nextName.equals("norwegianName")) {
				norwegianName = reader.nextString();
			} else if (nextName.equals("newNorwegianName")) {
				newNorwegianName = reader.nextString();
			} else if (nextName.equals("englishName")) {
				englishName = reader.nextString();
			} else if (nextName.equals("versionCode")) {
				versionCode = reader.nextInt();
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new Course(code, name, norwegianName, newNorwegianName, englishName, versionCode);
	}

}