package no.skarasoftwaredevelopment.schooltracker.pojo;

import java.io.Serializable;
import java.util.Calendar;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.pojo.activities.ActivityType;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Exercise;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Lecture;

public class Attendance implements Serializable {

	private static final long serialVersionUID = -5014671402300650260L;
	
	private int id;
	private Calendar startDate;
	private Calendar endDate;
	private int duration;
	private boolean attended;
	private boolean passed;
	private String courseCode;
	private String courseName;
	private ActivityType type;
	private List<Room> rooms;
	private boolean inside;
	
	public Attendance(int id, Calendar startDate, Calendar endDate, int duration, boolean attended, boolean passed, String courseCode, String courseName, ActivityType type, List<Room> rooms) {
		this.id = id;
		this.startDate = startDate;
		this.endDate = endDate;
		this.duration = duration;
		this.attended = attended;
		this.passed = passed;
		this.courseCode = courseCode;
		this.courseName = courseName;
		this.type = type;
		this.rooms = rooms;
	}
	
	// TODO: remove me, dummy constructor
	public Attendance(int id) {
		this.id = id;
	}
	
	public Attendance(int id, Calendar startDate, Calendar endDate, int duration, boolean attended, boolean passed) {
		this.id = id;
		this.startDate = startDate;
		this.endDate = endDate;
		this.duration = duration;
		this.attended = attended;
		this.passed = passed;
	}
	
	public Attendance(Calendar startDate, Calendar endDate, int duration, boolean attended, boolean passed) {
		this.startDate = startDate;
		this.endDate = endDate;
		this.duration = duration;
		this.attended = attended;
		this.passed = passed;
	}

	public int getId() {
		return id;
	}

	public Calendar getStartDate() {
		return startDate;
	}

	public Calendar getEndDate() {
		return endDate;
	}

	public int getDuration() {
		return duration;
	}

	public boolean hasAttended() {
		return attended;
	}
	
	public void setAttended(boolean attended) {
		this.attended = attended;
	}

	public boolean hasPassed() {
		return passed;
	}
	
	public void setPassed(boolean passed) {
		this.passed = passed;
	}

	public String getCourseCode() {
		return courseCode;
	}

	public String getCourseName() {
		return courseName;
	}

	public ActivityType getType() {
		return type;
	}

	public List<Room> getRooms() {
		return rooms;
	}
	
	public boolean isInside() {
		return inside;
	}
	
	public void setInside(boolean inside) {
		this.inside = inside;
	}

	@Override
	public String toString() {
		return "Attendance [id=" + id + ", startDate=" + startDate
				+ ", endDate=" + endDate + ", duration=" + duration
				+ ", attended=" + attended + ", passed=" + passed + "]";
	}
	
	public void checkAttendance(TaskData task) {
		if (getType() instanceof Lecture) {
			if (task.getPositionsInside() >= task.getThreshold()) {
				setAttended(true);
			}
		} else if (getType() instanceof Exercise) {
			if (task.getPositionsInside() >= Exercise.ATTENDANCE_THRESHOLD) {
				setAttended(true);
			}
		}
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Attendance))
			return false;
		Attendance other = (Attendance) obj;
		if (id != other.id)
			return false;
		return true;
	}

}