package no.skarasoftwaredevelopment.schooltracker.pojo;

import java.io.Serializable;

public class Coord implements Serializable {

	public static final double DUMMY_GEO_LON = -1000.0; // invalid scope for longitude
	public static final double DUMMY_GEO_LAT = -1000.0; // invalid scope for latitude
	
	private static final long serialVersionUID = 7161861910884784322L;
	private static final double EQUATORIAL_EARTH_RADIUS = 6378.1370D; // km
    private static final double DEGREE_TO_RADIAN = (Math.PI / 180D);
    
    private double longitude;
    private double latitude;
    private double x;
    private double y;
    private double z;
    
    public Coord(double longitude, double latitude) {
    	this.longitude = longitude;
    	this.latitude = latitude;
    }
    
    public Coord(double longitude, double latitude, double x, double y, double z) {
    	this.longitude = longitude;
    	this.latitude = latitude;
    	this.x = x;
    	this.y = y;
    	this.z = z;
    }

	public double getLongitude() {
		return longitude;
	}

    public double getLatitude() {
		return latitude;
	}
	
	public double getX() {
		return x;
	}
	
	public double getY() {
		return y;
	}
	
	public double getZ() {
		return z;
	}

	/**
	 * Calculate the distance between coords in meters
     * given GPS coordinates using the Haversine method.
     * 
	 * @param coord
	 * @return
	 */
    public int distanceInM(Coord coord) {
    	// Android built in method for calculating distance between coordinates
//    	float[] results = new float[3];
//    	Location.distanceBetween(latitude, longitude, coord.getLatitude(), coord.getLongitude(), results);
//    	return (int) results[0]; // meters
    	
        return (int) (1000D * distanceInKM(coord));
    }

    /**
     * Calculate the distance between coords in kilometers
     * given GPS coordinates using the Haversine method.
     * 
     * @param coord
     * @return
     */
    public double distanceInKM(Coord coord) {
        double dlong = (coord.getLongitude() - longitude) * DEGREE_TO_RADIAN;
        double dlat = (coord.getLatitude() - latitude) * DEGREE_TO_RADIAN;
        double a = Math.pow(Math.sin(dlat / 2D), 2D) + Math.cos(latitude * DEGREE_TO_RADIAN)
        		* Math.cos(coord.getLatitude() * DEGREE_TO_RADIAN) * Math.pow(Math.sin(dlong / 2D), 2D);
        double c = 2D * Math.atan2(Math.sqrt(a), Math.sqrt(1D - a));
        double d = EQUATORIAL_EARTH_RADIUS * c;

        return d;
    }

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(latitude);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(longitude);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(x);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(y);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(z);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Coord))
			return false;
		Coord other = (Coord) obj;
		if (Double.doubleToLongBits(latitude) != Double
				.doubleToLongBits(other.latitude))
			return false;
		if (Double.doubleToLongBits(longitude) != Double
				.doubleToLongBits(other.longitude))
			return false;
		if (Double.doubleToLongBits(x) != Double.doubleToLongBits(other.x))
			return false;
		if (Double.doubleToLongBits(y) != Double.doubleToLongBits(y))
			return false;
		if (Double.doubleToLongBits(z) != Double.doubleToLongBits(z))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Coord [longitude=" + longitude + ", latitude=" + latitude
				+ ", x=" + x + ", y=" + y + ", z=" + z + "]";
	}

}