package no.skarasoftwaredevelopment.schooltracker.pojo;

import java.io.Serializable;

public class PositionData implements Serializable {
	
	private static final long serialVersionUID = -9024797194673376395L;

	private String changedOn;
	private double confidenceFactor;
	private String elem;
	private Coord coord;
	private double x;
	private double y;
	private double z; // signifies the floor
	private PositionProvider provider;

	public enum PositionProvider {
		MAZEMAP, GOOGLE, DUMMY
	}
	
	/**
	 * This constructor is used by {@code PositionParser} when we
	 * get data about the position from MazeMap.
	 * 
	 * @param changedOn
	 * @param confidenceFactor
	 * @param elem
	 * @param geoLongitude
	 * @param geoLatitude
	 * @param x
	 * @param y
	 * @param z
	 */
	public PositionData(String changedOn, double confidenceFactor, String elem,
			double geoLongitude, double geoLatitude, double x, double y, double z) {
		this.changedOn = changedOn;
		this.confidenceFactor = confidenceFactor;
		this.elem = elem;
		this.coord = new Coord(geoLongitude, geoLatitude);
		this.x = x;
		this.y = y;
		this.z = z;
		provider = PositionProvider.MAZEMAP;
	}
	
	/**
	 * This constructor is used by {@code PositionParser} when we
	 * get data about the position from Google Play Services, and
	 * when we're generating a dummy position.
	 * 
	 * @param geoLongitude
	 * @param geoLatitude
	 */
	public PositionData(double geoLongitude, double geoLatitude) {
		this.changedOn = null;
		this.confidenceFactor = 0.0;
		this.elem = null;
		this.coord = new Coord(geoLongitude, geoLatitude);
		this.x = 0.0;
		this.y = 0.0;
		this.z = 0.0;
		provider = PositionProvider.GOOGLE;
	}
	
	/**
	 * This method is used in cases where neither MazeMap or
	 * Google is able to yield any positional data.
	 * 
	 * @return
	 */
	public static PositionData getDummy() {
		PositionData data = new PositionData(Coord.DUMMY_GEO_LON, Coord.DUMMY_GEO_LAT);
		data.setProvider(PositionProvider.DUMMY);
		return data;
	}

	public String getChangedOn() {
		return changedOn;
	}

	public double getConfidenceFactor() {
		return confidenceFactor;
	}

	public String getElem() {
		return elem;
	}

	public Coord getCoord() {
		return coord;
	}

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}

	public double getZ() {
		return z;
	}
	
	public PositionProvider getProvider() {
		return provider;
	}
	
	private void setProvider(PositionProvider provider) {
		this.provider = provider;
	}

}