package no.skarasoftwaredevelopment.schooltracker.pojo.activities;

import java.io.Serializable;

public abstract class ActivityType implements Serializable {
	
	public static final ActivityType LECTURE = new Lecture(""); // no type name required as this is a generic term
	public static final ActivityType LECTURE_REGULAR = new LectureRegular();
	public static final ActivityType LECTURE_EXERCISE = new LectureExercise();
	public static final ActivityType LECTURE_SEMINAR = new LectureSeminar();
	public static final ActivityType EXERCISE = new Exercise(""); // no type name required as this is a generic term
	public static final ActivityType EXERCISE_REGULAR = new ExerciseRegular();
	public static final ActivityType EXERCISE_LAB = new ExerciseLab();
	
	private static final long serialVersionUID = -7662887164574214178L;
	
	private static ActivityType[] types;
	
	private final String type;
	
	static {
		types = new ActivityType[5];
		types[0] = LECTURE_REGULAR;
		types[1] = LECTURE_EXERCISE;
		types[2] = LECTURE_SEMINAR;
		types[3] = EXERCISE_REGULAR;
		types[4] = EXERCISE_LAB;
	}
	
	public ActivityType(String type) {
		this.type = type;
	}
	
	public String toString() {
		return type;
	}

	public static ActivityType getType(String type) {
		// searches the activities we have
		// for a matching type string that
		// is either a lecture or an exercise
		for (ActivityType at : types) {
			if (type.equalsIgnoreCase(at.toString())) {
				return at;
			}
		}
		
		// the given type is not recognized,
		// return a class with the given string
		// to ensure that it shows up in the GUI
		return new Other(type);
	}

}