package no.skarasoftwaredevelopment.schooltracker.receivers;

import java.util.Calendar;

import no.skarasoftwaredevelopment.schooltracker.services.SchedulingService;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.support.v4.content.WakefulBroadcastReceiver;

/**
 * When the alarm fires, this {@code WakefulBroadcastReceiver} receives the broadcast {@code Intent} 
 * and then starts the {@code IntentService} {@code SchedulingService} to schedule alarms.
 */
public class AlarmReceiver extends WakefulBroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {   
        Intent service = new Intent(context, SchedulingService.class);
        // starts the service and keeps the device awake while it is launching
        startWakefulService(context, service);
    }

    /**
     * Sets a repeating alarm that runs once a day at approximately 03:00. When the
     * alarm fires, the app broadcasts an {@code Intent} to this {@code WakefulBroadcastReceiver}.
     * 
     * @param context
     */
    public void setAlarm(Context context) {
        // define the alarm's trigger time
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 3);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);

        // set the alarm
        AlarmManager alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
        
        Intent intent = new Intent(context, AlarmReceiver.class);
        PendingIntent alarmIntent = PendingIntent.getBroadcast(context, 0, intent, 0);
        
        alarmManager.setInexactRepeating(
        		AlarmManager.RTC_WAKEUP,
                calendar.getTimeInMillis(),
                AlarmManager.INTERVAL_DAY,
                alarmIntent);
    }

}