package no.skarasoftwaredevelopment.schooltracker.services.deprecated;

import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import android.app.IntentService;
import android.content.Intent;
import android.util.Log;

public class PositionServiceOld extends IntentService {

	public static final String TAG = "PositionServiceOld";
	public static final String EXTRA_ATTENDANCE = "no.skarasoftwaredevelopment.schooltracker.alarm.ATTENDANCE";
	public static final String EXTRA_STUDENT = "no.skarasoftwaredevelopment.schooltracker.alarm.STUDENT";
	public static final String EXTRA_COURSE_CODE = "no.skarasoftwaredevelopment.schooltracker.alarm.COURSE_CODE";
	
	private static List<PositionTaskOld> tasks = new ArrayList<PositionTaskOld>();
	
    public PositionServiceOld() {
        super(TAG);
    }
    
    /**
     * This method checks if the task has already been
     * added by checking if the list includes the new
     * taskId. The difference between this and task.contains()
     * is that we don't have to create the PositionTask
     * object first, which automatically starts a location
     * listener even though the task is already added.
     * This prevents us from having multiple unnecessary
     * listeners as a result of trying to add existing tasks.
     * 
     * @param taskId
     * @return
     */
    private static boolean taskExists(int taskId) {
    	for (PositionTaskOld task : tasks) {
    		if (task.getTaskId() == taskId) {
    			return true;
    		}
    	}
    	
    	return false;
    }
    
    /**
     * Add a new task only if it doesn't already exist.
     * 
     * @param position
     * @return
     */
    private void addTask(PositionTaskOld position) {
    	if (!tasks.contains(position)) {
    		tasks.add(position);
    	}
    }
    
    /**
     * Cancels and removes all tasks attached to a course.
     * given the course code.
     * @param courseCode
     */
    public static void cancelAllTasks(String courseCode) {
    	if (courseCode != null) {
	    	for (int i = tasks.size() - 1; i >= 0; i--) {
	    		PositionTaskOld task = tasks.get(i);	
	    		if (task.getAttendance().getCourseCode().equalsIgnoreCase(courseCode)) {
	    			cancelTask(task);
	    		}
	    	}
    	}
    }
    
    /**
     * Cancels and removes the given task.
     * 
     * @param position
     */
    public static void cancelTask(PositionTaskOld position) {
    	if (position != null) {
    		position.cancel();
    		tasks.remove(position);
    	}
    }
	
	/**
	 * Gets the number of tasks currently running.
	 * 
	 * @return size of tasks
	 */
	public static int getNumberOfTasksRunning() {
		return tasks.size();
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		Log.e(TAG, "creating intentservice..");
	}
	
	@Override
	public void onDestroy() {
		Log.e(TAG, "destroying intentservice..");
		super.onDestroy();
	}
    
	@Override
	protected void onHandleIntent(Intent intent) {
		Attendance attendance = (Attendance) intent.getSerializableExtra(EXTRA_ATTENDANCE);
		int taskId = attendance.getId();
		
		if (!taskExists(taskId)) {
			Log.e(TAG, "adding task (attendanceId: " + taskId + ")");
			
			Student student = (Student) intent.getSerializableExtra(EXTRA_STUDENT);
			PositionTaskOld task = new PositionTaskOld(getApplicationContext(), student.getId(), student.getUUID(), attendance);
			addTask(task);
			
			Thread thread = new Thread(task);
			thread.setPriority(Thread.MAX_PRIORITY);
			android.os.Process.setThreadPriority(android.os.Process.THREAD_PRIORITY_FOREGROUND);
			
			// executes the first run() that in turn will call itself
			thread.start();
			
			// the downside of this method is that we need to have a fixed size for the
			// thread pool which would mean that if the student has more activities than
			// the pool initially is able to maintain, we then need to recreate the pool
			// with all the same tasks again
			//ScheduledThreadPoolExecutor exec = new ScheduledThreadPoolExecutor(1);
			//exec.scheduleAtFixedRate(task, 0, Constants.POSITION_INTERVAL_MS, TimeUnit.MICROSECONDS);
		} else {
			Log.e(TAG, "task (attendanceId: " + taskId + ") is already added");
		}
	}
    
}