package no.skarasoftwaredevelopment.schooltracker.services.deprecated;

import java.io.Serializable;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TimerTask;

import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.LootHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.NotificationHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ReportPositionHandler;
import no.skarasoftwaredevelopment.schooltracker.parsers.PositionParser;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Item;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudentPosition;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Exercise;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Lecture;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.GoogleApiClient.ConnectionCallbacks;
import com.google.android.gms.common.api.GoogleApiClient.OnConnectionFailedListener;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

public class PositionTaskOld extends TimerTask implements Serializable, ConnectionCallbacks, OnConnectionFailedListener, LocationListener {

	public static final String TAG = "PositionTask";
    public static final String ACTION_BROADCAST = "no.skarasoftwaredevelopment.schooltracker.alarm.BROADCAST";
    public static final String EXTRA_ATTENDANCE = "no.skarasoftwaredevelopment.schooltracker.alarm.ATTENDANCE";
    
    private static final long serialVersionUID = -5214252296773834264L;
    
	private Context c;
	private int studentId;
	private String studentUUID;
	private int counter;
	private int taskId;
	private Attendance attendance;
	private DbHelper db;
	private int numberOfRuns;
	private int threshold;
	private int inside;
	private Location lastKnown;
	private GoogleApiClient googleApiClient;
	private LocationRequest locationRequest;
	private boolean hasPositionalData;
	
	public PositionTaskOld(Context c, int studentId, String studentUUID, Attendance attendance) {
		this.c = c;
		this.studentId = studentId;
		this.studentUUID = studentUUID;
		this.taskId = attendance.getId();
		this.attendance = attendance;
		
		// connect to google play services and create a
		// location request used by the LocationServices
		buildGoogleApiClient();
		createLocationRequest();
		
		counter = 0;
		db = new DbHelper(c);

//		// calculate number of runs based on the duration and interval
//		numberOfRuns = (int) Math.ceil((double) attendance.getDuration() / Constants.POSITION_INTERVAL_REGULAR_MIN);
//		//Log.e(TAG, "taskId (" + taskId + "): number of runs = " + numberOfRuns);
//		
//		// we need to account for the 15 minute breaks between each block during lectures
//		int breakMinutes = attendance.getDuration() % Constants.LECTURE_BLOCK_LENGTH;
//		int breakSkips = (int) Math.ceil((double) breakMinutes / Constants.POSITION_INTERVAL_REGULAR_MIN);
//		
//		// calculate the threshold for the lowest amount of positions required by the app
//		// to be considered inside by the app. this is only applicable for lecture type activities
//		threshold = (int) Math.ceil((double) (numberOfRuns * Constants.ATTENDANCE_THRESHOLD) / 100);
//
//		// we don't want to punish the student for leaving the designated room(s)
//		// during a break which is why we subtract the breaks from the threshold
//		threshold -= breakSkips;
//		//Log.e(TAG, "taskId (" + taskId + "): threshold = " + threshold);
		
		Log.e(TAG, "starting task (" + taskId + ")..");
	}
	
	public void run() {
		Calendar now = Calendar.getInstance(new Locale("NO"));

		if (counter < numberOfRuns && now.after(attendance.getStartDate()) && now.before(attendance.getEndDate())) {
			Log.e(TAG, "taskId (" + taskId + "): running task.. (" + (counter + 1) + "/" + numberOfRuns + ")");
			counter++;

			// we try to get a position from MazeMap first
			PositionData pos = PositionParser.parsePosition();
			
			// if MazeMap/MSE failed us, we need to try and
			// get a position from the LocationManager
			if (pos != null || googleApiClient.isConnected()) {				
				if (lastKnown != null && System.currentTimeMillis() - lastKnown.getTime() <= (ConstantHandler.POSITION_INTERVAL_REGULAR_MS * 2)) {
					// create PositionData from Location
					pos = new PositionData(null, 0.0, null, lastKnown.getLongitude(), lastKnown.getLatitude(), 0.0, 0.0, 0.0);
					hasPositionalData = true;
				} else {
					// there isn't a location available (turned off the gps?) so we need
					// to create a new dummy position to avoid null pointers below
					pos = new PositionData(null, 0.0, null, -1.0, -1.0, 0.0, 0.0, 0.0);
					hasPositionalData = false;
					Log.e(TAG, "no last known location..");
				}
			} else {
				// we don't have any positional data whatsoever
				// so we create a new dummy position to avoid
				// null pointers below
				pos = new PositionData(null, 0.0, null, -1.0, -1.0, 0.0, 0.0, 0.0);
				hasPositionalData = false;
				Log.e(TAG, "no positional data available..");
			}

			// no reason to check if the postion is inside a given room
			// when we know for a fact that the position is a dummy position
			if (hasPositionalData) {
				List<Room> rooms = attendance.getRooms();
				for (Room r : rooms) {
					// we only need to know that the student is inside one of the possible rooms
					attendance.setInside(r.isPositionInside(pos.getCoord()));
					if (attendance.isInside()) {
						inside++;
						break;
					}
				}
			}
			
			Log.e(TAG, "getting position for " + attendance.getCourseCode() + " (taskId: " + taskId + ").." + attendance.isInside());
			
			// helps encapsulate all the data we wish to pass on to the
			// two methods below
			StudentPosition studentPos = new StudentPosition(
					studentId, attendance.getId(),
					String.valueOf(pos.getCoord().getLongitude()),
					String.valueOf(pos.getCoord().getLatitude()),
					pos.getZ(), attendance.isInside(), pos.getProvider());			

			// TODO: should we use this code for checking if a position
			// is on the campus (Glshaugen) or not
//			Coord[] geoBoundary = new Coord[4];
//			geoBoundary[0] = null; // bottom left
//			geoBoundary[1] = null; // top left
//			geoBoundary[2] = null; // top right
//			geoBoundary[3] = null; // bottom right
//			Room ntnuBoundary = new Room(-1, null, null, null, -1, -1, null, geoBoundary, -1);
//			if (ntnuBoundary.isPositionInside(pos.getCoord()))

			// saves the current position (in case we need it later)
			// as well as the attendance the position is connected to
			// and if the position was inside the room(s) or not
			db.addStudentPosition(studentPos);
			ReportPositionHandler.report(studentUUID, attendance.getCourseCode(), studentPos);

			Intent localIntent = new Intent(ACTION_BROADCAST).putExtra(EXTRA_ATTENDANCE, attendance);
			LocalBroadcastManager.getInstance(c).sendBroadcast(localIntent);
		} else {
			// saves the result based on positions collected during the activity 
			attendance.setPassed(true);
			
			// if the counter is smaller than the threshold then that means
			// either the app was restarted during the task or the course
			// was added during the task
			if (counter < threshold) {
				inside = db.getStudentPositionsInside(1, attendance.getId()); // TODO: get studentId dynamically
			}
			
			// we require students to attend a set portion of the lectures
			// for them to count as attended, but for exercises they only need
			// one position to be inside for it to count
			if (attendance.getType() instanceof Lecture) {
				if (inside >= threshold) {
					attendance.setAttended(true);
				}
			} else if (attendance.getType() instanceof Exercise) {
				if (inside > 0) {
					attendance.setAttended(true);
				}
			}
			
			// the student has a chance to generate loot
			// if he has attended the activity
			if (attendance.hasAttended()) {
				Item loot = LootHandler.generateLoot();
				if (loot != null) {
					db.addLoot(studentId, loot); // TODO: should get studentId dynamically
					NotificationHandler.notifyLoot(c);
				}
				
				// check for and add new achievements,
				// and then notify the user with a notification
				List<Achievement> achievements = AchievementHandler.checkAchievements(c, 1); // TODO: get studentId dynamically 
				
				if (achievements.size() > 0) {
					db.addStudentAchievements(1, achievements); // TODO: dynamically find studentId
					NotificationHandler.notifyAchievement(c, achievements.size());
				}
			}
			
			// TODO: enable the line below to save status about the attendance based on the positions
			//db.updateAttendance(1, attendance.getId(), attendance.hasAttended(), attendance.hasPassed()); // TODO: should get studentId dynamically
			
			// forces the GUI to refresh the effort data for all the courses 
			db.setStudentEffortUpToDate(1, false); // TODO: should get studentId dynamically
			// forces the GUI to refresh the effort data for the given course
			db.setStudentCourseEffortUpToDate(1, attendance.getCourseCode(), false); // TODO: should get studentId dynamically
			
			Intent localIntent = new Intent(ACTION_BROADCAST).putExtra(EXTRA_ATTENDANCE, attendance);
			LocalBroadcastManager.getInstance(c).sendBroadcast(localIntent);
			
			Log.e(TAG, "status (" + taskId + "): " + inside + " / " + threshold + " (" + numberOfRuns + ")");
			Log.e(TAG, "stopping task (" + taskId + ")..");

			PositionServiceOld.cancelTask(this); // TODO: comment out this if used again
		}
	}
	
	@Override
	public boolean cancel() {
		if (googleApiClient.isConnected()) {
			LocationServices.FusedLocationApi.removeLocationUpdates(googleApiClient, this);
			googleApiClient.disconnect();
		}
		
		return super.cancel();
	}
	
	public int getTaskId() {
		return taskId;
	}
	
	public Attendance getAttendance() {
		return attendance;
	}
	
	private synchronized void buildGoogleApiClient() {
		googleApiClient = new GoogleApiClient.Builder(c)
	        .addConnectionCallbacks(this)
	        .addOnConnectionFailedListener(this)
	        .addApi(LocationServices.API)
	        .build();
		googleApiClient.connect();
	}
	
	private void createLocationRequest() {
		// creates a request that decides interval/priority
		// of when location updates should take place
		locationRequest = new LocationRequest();
		locationRequest.setInterval(ConstantHandler.POSITION_INTERVAL_REGULAR_MS);
		//locationRequest.setFastestInterval(ConstantHandler.POSITION_INTERVAL_FASTEST_MS);
		
		// not sure if this one is accurate enough, according to Google
		// this priority has an accuracy of approximately 100 meters
		// (https://developer.android.com/training/location/receive-location-updates.html)
		// which in most cases isn't good enough for a classroom, however
		// testing has shown very promising results
		//locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY);
		locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
	}
	
	@Override
	public void onConnected(Bundle bundle) {
        LocationServices.FusedLocationApi.requestLocationUpdates(googleApiClient, locationRequest, this);
        // in case the device has recently requested a location nearby
        // then we try to use this before the LocationListener starts
        // getting positions
        lastKnown = LocationServices.FusedLocationApi.getLastLocation(googleApiClient);
	}
	
	@Override
	public void onLocationChanged(Location location) {
		Log.e(TAG + "/LOCATION", "new position..");
		lastKnown = location;
	}
	
	@Override
	public void onConnectionFailed(ConnectionResult result) {
		Log.e(TAG + "/LOCATION", "connection failed..");
	}

	@Override
	public void onConnectionSuspended(int error) {
		Log.e(TAG + "/LOCATION", "connection suspended..");
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + taskId;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof PositionTaskOld))
			return false;
		PositionTaskOld other = (PositionTaskOld) obj;
		if (taskId != other.taskId)
			return false;
		return true;
	}
	
}