package no.skarasoftwaredevelopment.schooltracker.services.deprecated;

import java.io.Serializable;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.LootHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.NotificationHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ReportPositionHandler;
import no.skarasoftwaredevelopment.schooltracker.parsers.PositionParser;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Item;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudentPosition;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Exercise;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.Lecture;
import no.skarasoftwaredevelopment.schooltracker.services.PositionService;
import android.content.Intent;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;

public class PositionTaskOld2 implements Serializable, Runnable {//, ConnectionCallbacks, OnConnectionFailedListener, LocationListener, Runnable {
	
	public static final String TAG = "PositionTask";
    public static final String ACTION_BROADCAST = "no.skarasoftwaredevelopment.schooltracker.alarm.BROADCAST";
    public static final String EXTRA_ATTENDANCE = "no.skarasoftwaredevelopment.schooltracker.alarm.ATTENDANCE";

    private static final long serialVersionUID = 951105969340827617L;

    private Thread thread;
	private PositionService service;
	private int studentId;
	private String studentUUID;
	private int counter;
	private int taskId;
	private Attendance attendance;
	private DbHelper db;
	private int numberOfRuns;
	private int threshold;
	private int inside;
	//private Location lastKnown;
	//private GoogleApiClient googleApiClient;
	//private LocationRequest locationRequest;
	private boolean hasPositionalData;
	private boolean canceled;
	private Calendar now;
	
	private int test;
	
	public void incrementTest() {
		test++;
	}
	
	public int getTest() {
		return test;
	}
	
	public PositionTaskOld2(PositionService service, int studentId, String studentUUID, Attendance attendance) {
		this.service = service;
		this.studentId = studentId;
		this.studentUUID = studentUUID;
		this.taskId = attendance.getId();
		this.attendance = attendance;
		
		// connect to google play services and create a
		// location request used by the LocationServices
		//buildGoogleApiClient();
		//createLocationRequest();
		
		counter = 0;
		canceled = false;
		db = new DbHelper(service);
	
		// calculate number of runs based on the duration and interval
		// we need to add one additional run to the total otherwise
		// each task would stop Constants.POSITION_INTERVAL_REGULAR_MIN
		// minutes before the activity actually ends
		numberOfRuns = (int) Math.ceil((double) attendance.getDuration() / (double) ConstantHandler.POSITION_INTERVAL_REGULAR_MIN) + 1;
		Log.e(TAG, "taskId (" + taskId + "): number of runs = " + numberOfRuns);
		
		// we cast the result of the division to an int to cut off the decimals (effectively rounding down)
		// there is no need to take into account the extra Constants.POSITION_INTERVAL_REGULAR_MIN minutes
		// here as an activity never will end this close to it's next pause anyhow
		int numberOfBreaks = (int) (attendance.getDuration() / ConstantHandler.LECTURE_BLOCK_INCLUDING_BREAK);
		int breaksInRuns = (numberOfBreaks * ConstantHandler.BREAK_LENGTH) / ConstantHandler.POSITION_INTERVAL_REGULAR_MIN;

		// we need to account for the breaks between each lecture block since we don't want to punish the
		// student for leaving the designated room(s) during the activity's life span. we only require the
		// student to attend 75% of the lecture, which is also included in this calculation.
		int runsNoBreaks = numberOfRuns - breaksInRuns;

		// calculate the threshold for the lowest amount of positions required for the student to be
		// considered as attended by the app. this is only applicable to lecture type activities.
		threshold = (int) Math.ceil((double) runsNoBreaks * ConstantHandler.ATTENDANCE_THRESHOLD_FACTOR);
		Log.e(TAG, "taskId (" + taskId + "): threshold = " + threshold);		
		
		Log.e(TAG, "starting task (" + taskId + ")..");
	}
	
	private boolean taskStillRunning() {
		now = Calendar.getInstance(new Locale("NO"));
		return counter < numberOfRuns && !canceled && now.after(attendance.getStartDate()) && now.before(attendance.getEndDate());
	}
	
	@Override
	public void run() {		
		// we need a reference to the thread in the task so the task
		// itself is able to stop the thread when it's done
		thread = Thread.currentThread();
	
		// due to the nature of threads and thread handling, we need to account for slight
		// variances in the run times of threads, meaning that the last thread need to be
		// able to run a few seconds/minutes after the activity ended. this will not cause
		// any issues with the number of times the task is run due to the counter we have.
		attendance.getEndDate().add(Calendar.MINUTE, ConstantHandler.POSITION_INTERVAL_REGULAR_MIN);
		
		long time = 0;
		if (taskStillRunning()) {
			time = System.currentTimeMillis();
			
			// sets this thread's priority
			thread.setPriority(Thread.MAX_PRIORITY);
			android.os.Process.setThreadPriority(android.os.Process.THREAD_PRIORITY_FOREGROUND);
			
			Log.e(TAG, "taskId (" + taskId + "): running task.. (" + (counter + 1) + "/" + numberOfRuns + ")");
			counter++;
	
			// we try to get a position from MazeMap first
			PositionData pos = PositionParser.parsePosition();

//			if (pos == null) {
//				// if MazeMap/MSE failed us, we need to try and get a position from the LocationManager
//				if (googleApiClient.isConnected()) {
//					if (lastKnown != null && System.currentTimeMillis() - lastKnown.getTime() <= (ConstantHandler.POSITION_INTERVAL_REGULAR_MS * 2)) {
//						// create PositionData from Location
//						pos = new PositionData(lastKnown.getLongitude(), lastKnown.getLatitude());
//						hasPositionalData = true;
//						Log.e(TAG, "taskId (" + taskId + "): GPS location found..");
//					} else {
//						// there isn't a location available (turned off the gps?) so we need
//						// to request a dummy position to avoid null pointers below
//						pos = PositionData.getDummy();
//						hasPositionalData = false;
//						Log.e(TAG, "taskId (" + taskId + "): no last known GPS location..");
//					}
//				} else {
//					// we don't have any positional data whatsoever so we request a dummy
//					// position to avoid null pointers below
//					pos = PositionData.getDummy();
//					hasPositionalData = false;
//					Log.e(TAG, "taskId (" + taskId + "): no positional data available..");
//				}
//			} else {
//				// we succeeded in getting a position from MazeMap
//				Log.e(TAG, "taskId (" + taskId + "): MazeMap location found..");
//				hasPositionalData = true;
//			}
	
			// no reason to check if the position is inside a given room
			// when we know for a fact that the position is a dummy position
			if (hasPositionalData) {
				List<Room> rooms = attendance.getRooms();
				for (Room r : rooms) {
					// we only need to know that the student is inside one of the possible rooms
					attendance.setInside(r.isPositionInside(pos.getCoord()));
					if (attendance.isInside()) {
						inside++;
						break;
					}
				}
			}
			
			Log.e(TAG, "getting position for " + attendance.getCourseCode() + " (taskId: " + taskId + ").." + attendance.isInside());
			
			// helps encapsulate all the data we wish to pass on to the
			// two methods below
			StudentPosition studentPos = new StudentPosition(
					studentId, attendance.getId(),
					String.valueOf(pos.getCoord().getLongitude()),
					String.valueOf(pos.getCoord().getLatitude()),
					pos.getZ(), attendance.isInside(), pos.getProvider());			
	
			// TODO: should we use this code for checking if a position
			// is on the campus (Glshaugen) or not
	//		Coord[] geoBoundary = new Coord[4];
	//		geoBoundary[0] = null; // bottom left
	//		geoBoundary[1] = null; // top left
	//		geoBoundary[2] = null; // top right
	//		geoBoundary[3] = null; // bottom right
	//		Room ntnuBoundary = new Room(-1, null, null, null, -1, -1, null, geoBoundary, -1);
	//		if (ntnuBoundary.isPositionInside(pos.getCoord()))
	
			// saves the current position (in case we need it later)
			// as well as the attendance the position is connected to
			// and if the position was inside the room(s) or not
			db.addStudentPosition(studentPos);
			ReportPositionHandler.report(studentUUID, attendance.getCourseCode(), studentPos);
	
			Intent localIntent = new Intent(ACTION_BROADCAST).putExtra(EXTRA_ATTENDANCE, attendance);
			LocalBroadcastManager.getInstance(service).sendBroadcast(localIntent);
			
			
			Log.e(TAG, "execution time, pre pause: " + (System.currentTimeMillis() - time));
			
			// there is no reason to sleep the thread if this was the last
			// run needed to complete the task, if the task was canceled,
			// or if the time ran out (we're no longer in a valid interval)
			if (taskStillRunning()) {
				try {
					Thread.sleep(ConstantHandler.POSITION_INTERVAL_REGULAR_MS);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			Log.e(TAG, "execution time, post pause: " + (System.currentTimeMillis() - time));
		} else {
		
		// when we exit the while loop that means the work is done and that
		// the attendance has passed
		attendance.setPassed(true);
		
		// if the counter is smaller than the threshold then that means
		// either the app was restarted during the task or the course
		// was added during the task, and in either case it's better to
		// relay on the data from the database instead of this class
		if (counter < threshold) {
			inside = db.getStudentPositionsInside(1, attendance.getId()); // TODO: get studentId dynamically
		}
		
		// we require students to attend a set portion of the lectures
		// for them to count as attended, but for exercises they only need
		// one position to be inside for it to count
		if (attendance.getType() instanceof Lecture) {
			if (inside >= threshold) {
				attendance.setAttended(true);
			}
		} else if (attendance.getType() instanceof Exercise) {
			if (inside > 0) {
				attendance.setAttended(true);
			}
		}

		// saves the result/status about the attendance based on positions collected during the activity
		//db.updateAttendance(1, attendance.getId(), attendance.hasAttended(), attendance.hasPassed()); // TODO: should get studentId dynamically
		
		// forces the GUI to refresh the effort data for all the courses 
		db.setStudentEffortUpToDate(1, false); // TODO: should get studentId dynamically
		
		// forces the GUI to refresh the effort data for the given course
		db.setStudentCourseEffortUpToDate(1, attendance.getCourseCode(), false); // TODO: should get studentId dynamically
		
		// the student has a chance to generate loot
		// if he has attended the activity
		if (attendance.hasAttended()) {
			Item loot = LootHandler.generateLoot();
			if (loot != null) {
				db.addLoot(studentId, loot); // TODO: should get studentId dynamically
				NotificationHandler.notifyLoot(service);
			}
			
			// check for and add new achievements,
			// and then notify the user with a notification
			List<Achievement> achievements = AchievementHandler.checkAchievements(service, 1); // TODO: get studentId dynamically 
			
			if (achievements.size() > 0) {
				db.addStudentAchievements(1, achievements); // TODO: dynamically find studentId
				NotificationHandler.notifyAchievement(service, achievements.size());
			}
		}
		
		Intent localIntent = new Intent(ACTION_BROADCAST).putExtra(EXTRA_ATTENDANCE, attendance);
		LocalBroadcastManager.getInstance(service).sendBroadcast(localIntent);
		
		Log.e(TAG, "status (" + taskId + "): " + inside + " / " + threshold + " (" + numberOfRuns + ")");
		Log.e(TAG, "stopping task (" + taskId + ")..");
		
		//service.cancelTask(this);
		}
	}

	public void cancel() {
		canceled = true;
		
//		if (googleApiClient != null && googleApiClient.isConnected()) {
//			LocationServices.FusedLocationApi.removeLocationUpdates(googleApiClient, this);
//			googleApiClient.disconnect();
//		}
		
		if (thread != null) {
			thread.interrupt();
			thread = null;
		}
	}
	
	public int getTaskId() {
		return taskId;
	}
	
	public Attendance getAttendance() {
		return attendance;
	}
	
	public void setThread(Thread thread) {
		this.thread = thread;
	}
	
//	private synchronized void buildGoogleApiClient() {
//		googleApiClient = new GoogleApiClient.Builder(service)
//	        .addConnectionCallbacks(this)
//	        .addOnConnectionFailedListener(this)
//	        .addApi(LocationServices.API)
//	        .build();
//		googleApiClient.connect();
//	}
//	
//	private void createLocationRequest() {
//		// creates a request that decides interval/priority
//		// of when location updates should take place
//		locationRequest = new LocationRequest();
//		locationRequest.setInterval(ConstantHandler.POSITION_INTERVAL_REGULAR_MS);
//		locationRequest.setFastestInterval(ConstantHandler.POSITION_INTERVAL_FASTEST_MS);
//		
//		// not sure if this one is accurate enough, according to Google
//		// this priority has an accuracy of approximately 100 meters
//		// (https://developer.android.com/training/location/receive-location-updates.html)
//		// which in most cases isn't good enough for a classroom, however
//		// testing has shown very promising results
//		//locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY);
//		
//		// further testing of the above method has shown discrepancies in the results
//		// which led us to using this method instead
//		locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
//	}
//	
//	@Override
//	public void onConnected(Bundle bundle) {
//		if (googleApiClient != null && locationRequest != null) {
//		    LocationServices.FusedLocationApi.requestLocationUpdates(googleApiClient, locationRequest, this);
//		    
//		    // in case the device has recently requested a location nearby
//		    // then we try to use this before the LocationListener starts
//		    // getting positions
//		    lastKnown = LocationServices.FusedLocationApi.getLastLocation(googleApiClient);
//		}
//	}
	
//	@Override
//	public void onLocationChanged(Location location) {
//		Log.e(TAG + "/LOCATION", "taskId (" + taskId + "): new position..");
//		lastKnown = location;
//	}
//	
//	@Override
//	public void onConnectionFailed(ConnectionResult result) {
//		Log.e(TAG + "/LOCATION", "taskId (" + taskId + "): connection failed..");
//	}
//	
//	@Override
//	public void onConnectionSuspended(int error) {
//		Log.e(TAG + "/LOCATION", "taskId (" + taskId + "): connection suspended..");
//	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + taskId;
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof PositionTaskOld2))
			return false;
		PositionTaskOld2 other = (PositionTaskOld2) obj;
		if (taskId != other.taskId)
			return false;
		return true;
	}
	
}